@@ -4,9 +4,15 @@
 
 Components.utils.import("resource://calendar/modules/calUtils.jsm");
 Components.utils.import("resource://calendar/modules/calAlarmUtils.jsm");
+Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/Preferences.jsm");
+Components.utils.import("resource://gre/modules/Promise.jsm");
+Components.utils.import("resource://gre/modules/PromiseUtils.jsm");
 
 const kHoursBetweenUpdates = 6;
+const kSleepMonitorInterval = 60000;
+const kSleepMonitorTolerance = 1000;
 
 function nowUTC() {
     return cal.jsDateToDateTime(new Date()).getInTimezone(cal.UTC());
@@ -95,6 +101,11 @@ function calAlarmService() {
     };
 }
 
+const calAlarmServiceClassID = Components.ID("{7a9200dd-6a64-4fff-a798-c5802186e2cc}");
+const calAlarmServiceInterfaces = [
+    Components.interfaces.calIAlarmService,
+    Components.interfaces.nsIObserver
+];
 calAlarmService.prototype = {
     mRangeStart: null,
     mRangeEnd: null,
@@ -104,33 +115,15 @@ calAlarmService.prototype = {
     mObservers: null,
     mTimezone: null,
 
-    getInterfaces: function cAS_getInterfaces(aCount) {
-        let ifaces = [
-            Components.interfaces.nsISupports,
-            Components.interfaces.calIAlarmService,
-            Components.interfaces.nsIObserver,
-            Components.interfaces.nsIClassInfo
-        ];
-        aCount.value = ifaces.length;
-        return ifaces;
-    },
-
-    getHelperForLanguage: function cAS_getHelperForLanguage(language) {
-        return null;
-    },
-
-    /**
-     * nsIClassInfo
-     */
-    contractID: "@mozilla.org/calendar/alarm-service;1",
-    classDescription: "Calendar Alarm Service",
-    classID: Components.ID("{7a9200dd-6a64-4fff-a798-c5802186e2cc}"),
-    implementationLanguage: Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT,
-    flags: Components.interfaces.nsIClassInfo.SINGLETON,
-
-    QueryInterface: function cAS_QueryInterface(aIID) {
-        return doQueryInterface(this, calAlarmService.prototype, aIID, null, this);
-    },
+    classID: calAlarmServiceClassID,
+    QueryInterface: XPCOMUtils.generateQI(calAlarmServiceInterfaces),
+    classInfo: XPCOMUtils.generateCI({
+        classID: calAlarmServiceClassID,
+        contractID: "@mozilla.org/calendar/alarm-service;1",
+        classDescription: "Calendar Alarm Service",
+        interfaces: calAlarmServiceInterfaces,
+        flags: Components.interfaces.nsIClassInfo.SINGLETON
+    }),
 
     /**
      * nsIObserver
@@ -220,13 +213,9 @@ calAlarmService.prototype = {
             return;
         }
 
-        let observerSvc = Components.classes["@mozilla.org/observer-service;1"]
-                          .getService
-                          (Components.interfaces.nsIObserverService);
-
-        observerSvc.addObserver(this, "profile-after-change", false);
-        observerSvc.addObserver(this, "xpcom-shutdown", false);
-        observerSvc.addObserver(this, "wake_notification", false);
+        Services.obs.addObserver(this, "profile-after-change", false);
+        Services.obs.addObserver(this, "xpcom-shutdown", false);
+        Services.obs.addObserver(this, "wake_notification", false);
 
         /* Tell people that we're alive so they can start monitoring alarms.
          */
@@ -249,7 +238,7 @@ calAlarmService.prototype = {
                 if (!this.alarmService.mRangeEnd) {
                     // This is our first search for alarms.  We're going to look for
                     // alarms +/- 1 month from now.  If someone sets an alarm more than
-                    // a month ahead of an event, or doesn't start Sunbird/Lightning
+                    // a month ahead of an event, or doesn't start Lightning
                     // for a month, they'll miss some, but that's a slim chance
                     start = now.clone();
                     start.month -= 1;
@@ -302,12 +291,9 @@ calAlarmService.prototype = {
 
         this.mRangeEnd = null;
 
-        let observerSvc = Components.classes["@mozilla.org/observer-service;1"]
-                          .getService(Components.interfaces.nsIObserverService);
-
-        observerSvc.removeObserver(this, "profile-after-change");
-        observerSvc.removeObserver(this, "xpcom-shutdown");
-        observerSvc.removeObserver(this, "wake_notification");
+        Services.obs.removeObserver(this, "profile-after-change");
+        Services.obs.removeObserver(this, "xpcom-shutdown");
+        Services.obs.removeObserver(this, "wake_notification");
 
         this.mStarted = false;
     },
@@ -328,7 +314,7 @@ calAlarmService.prototype = {
             return;
         }
 
-        let showMissed = cal.getPrefSafe("calendar.alarms.showmissed", true);
+        let showMissed = Preferences.get("calendar.alarms.showmissed", true);
 
         let alarms = aItem.getAlarms({});
         for each (let alarm in alarms) {
@@ -496,13 +482,21 @@ calAlarmService.prototype = {
     findAlarms: function cAS_findAlarms(aCalendars, aStart, aUntil) {
         let getListener = {
             alarmService: this,
+            addRemovePromise: PromiseUtils.defer(),
             onOperationComplete: function cAS_fA_onOperationComplete(aCalendar,
                                                                      aStatus,
                                                                      aOperationType,
                                                                      aId,
                                                                      aDetail) {
-                // calendar has been loaded, so until now, onLoad events can be ignored:
-                this.alarmService.mLoadedCalendars[aCalendar.id] = true;
+                this.addRemovePromise.promise.then((aValue) => {
+                    // calendar has been loaded, so until now, onLoad events can be ignored:
+                    this.alarmService.mLoadedCalendars[aCalendar.id] = true;
+
+                    // notify observers that the alarms for the calendar have been loaded
+                    this.alarmService.mObservers.notify("onAlarmsLoaded", [aCalendar]);
+                }, function onReject(aReason) {
+                    Components.utils.reportError("Promise was rejected: " + aReason);
+                });
             },
             onGetResult: function cAS_fA_onGetResult(aCalendar,
                                                      aStatus,
@@ -510,12 +504,17 @@ calAlarmService.prototype = {
                                                      aDetail,
                                                      aCount,
                                                      aItems) {
-                for each (let item in aItems) {
-                    // assure we don't fire alarms twice, handle removed alarms as far as we can:
-                    // e.g. we cannot purge removed items from ics files. XXX todo.
-                    this.alarmService.removeAlarmsForItem(item);
-                    this.alarmService.addAlarmsForItem(item);
-                }
+                let promise = this.addRemovePromise;
+                cal.forEach(aItems, (item) => {
+                    try {
+                        this.alarmService.removeAlarmsForItem(item);
+                        this.alarmService.addAlarmsForItem(item);
+                    } catch (ex) {
+                        promise.reject(ex);
+                    }
+                }, function completed() {
+                    promise.resolve();
+                });
             }
         };
 
@@ -542,7 +541,7 @@ calAlarmService.prototype = {
 
         // Total refresh similar to startup.  We're going to look for
         // alarms +/- 1 month from now.  If someone sets an alarm more than
-        // a month ahead of an event, or doesn't start Sunbird/Lightning
+        // a month ahead of an event, or doesn't start Lightning
         // for a month, they'll miss some, but that's a slim chance
         let start = nowUTC();
         let until = start.clone();
