@@ -303,9 +303,8 @@ function deleteCalendars(controller, name){
   let cals = manager.getCalendars({});
 
   for (let i = 0; i < cals.length; i++)
-    if (cals[i].name == name){
-      manager.unregisterCalendar(cals[i]);
-      manager.deleteCalendar(cals[i]);
+    if (cals[i].name == name) {
+        manager.removeCalendar(cals[i]);
     }
 }
 
@@ -476,7 +475,7 @@ function setData(controller, data) {
   
   // timezone
   if (data.timezone != undefined) {
-    let menuitem = new elementslib.ID(controller.window.document, "options-timezone-menuitem");
+    let menuitem = new elementslib.ID(controller.window.document, "options-timezones-menuitem");
     menuitem.getNode().setAttribute("checked", data.timezone);
     controller.click(menuitem);
   }
