@@ -9,16 +9,15 @@ function run_test() {
 
     let dateTime1950 = cal.createDateTime();
     dateTime1950.year = 1950;
-    do_check_eq(dateTime1950.year, 1950);
+    equal(dateTime1950.year, 1950);
 
-    let dateTime1955 = cal.createDateTime();
-    dateTime1955.jsDate = new Date(1955, 06, 15);
-    do_check_eq(dateTime1955.year, 1955);
+    let dateTime1955 = cal.jsDateToDateTime(new Date(1955, 06, 15));
+    equal(dateTime1955.year, 1955);
     
     let dateTime1965 = cal.createDateTime();
     dateTime1965.nativeTime = -150000000000000;
-    do_check_eq(dateTime1965.year, 1965);
-    do_check_eq(dateTime1965.nativeTime, -150000000000000);
+    equal(dateTime1965.year, 1965);
+    equal(dateTime1965.nativeTime, -150000000000000);
 
     let dateTime1990 = cal.createDateTime();
     dateTime1990.year = 1990;
@@ -26,8 +25,8 @@ function run_test() {
     let dateTime2050 = cal.createDateTime();
     dateTime2050.year = 2050;
 
-    do_check_true(dateTime1950.nativeTime < dateTime1955.nativeTime);
-    do_check_true(dateTime1955.nativeTime < dateTime1965.nativeTime);
-    do_check_true(dateTime1965.nativeTime < dateTime1990.nativeTime);
-    do_check_true(dateTime1990.nativeTime < dateTime2050.nativeTime);
+    ok(dateTime1950.nativeTime < dateTime1955.nativeTime);
+    ok(dateTime1955.nativeTime < dateTime1965.nativeTime);
+    ok(dateTime1965.nativeTime < dateTime1990.nativeTime);
+    ok(dateTime1990.nativeTime < dateTime2050.nativeTime);
 }
