@@ -10,45 +10,41 @@
 # Repo Defaults
 # 'REV' controls the default rev for All the various repo's
 # Define x_REV to override. Where x can be one of:
-#  "COMM", "MOZILLA", "CHATZILLA", "INSPECTOR", "VENKMAN", "LDAPSDKS"
+#  "COMM", "MOZILLA", "CHATZILLA", "INSPECTOR", "LDAPSDKS"
 DEFAULTS = {
-  # Global Default Revision
-  'REV': "default",
+    # Global Default Revision
+    'REV': "default",
 
-  # LDAPSDKS
-  'LDAPSDKS_REPO': 'http://hg.mozilla.org/projects/ldap-sdks/',
-  'LDAPSDKS_REV': 'LDAPCSDK_6_0_7F_RTM',
+    # LDAPSDKS
+    'LDAPSDKS_REPO': 'https://hg.mozilla.org/projects/ldap-sdks/',
+    'LDAPSDKS_REV': 'LDAPCSDK_6_0_7H_RTM',
 
-  # URL of the default hg repository to clone for ChatZilla.
-  'CHATZILLA_REPO': 'http://hg.mozilla.org/chatzilla/',
-  # The stable revision to use
-  'CHATZILLA_REV':  'SEA2_21_RELBRANCH',
+    # URL of the default hg repository to clone for ChatZilla.
+    'CHATZILLA_REPO': 'https://hg.mozilla.org/chatzilla/',
+    # The stable revision to use
+      'CHATZILLA_REV':  'SEA2_32_RELBRANCH',
 
-  # URL of the default hg repository to clone for DOM Inspector.
-  'INSPECTOR_REPO': 'http://hg.mozilla.org/dom-inspector/',
-  # The stable revision to use
-  'INSPECTOR_REV':  'SEA2_21_RELBRANCH',
+    # URL of the default hg repository to clone for DOM Inspector.
+    'INSPECTOR_REPO': 'https://hg.mozilla.org/dom-inspector/',
+    # The stable revision to use
+      'INSPECTOR_REV':  'DOMI_LATEST_RELEASE',
 
-  # URL of the default hg repository to clone for Venkman.
-  'VENKMAN_REPO': 'http://hg.mozilla.org/venkman/',
-  # The stable revision to use
-  'VENKMAN_REV':  'SEA2_21_RELBRANCH',
-
-  # URL of the default hg repository to clone for Mozilla.
-  'MOZILLA_REPO': 'http://hg.mozilla.org/releases/mozilla-esr24/',
+    # URL of the default hg repository to clone for Mozilla.
+    'MOZILLA_REPO': 'https://hg.mozilla.org/releases/mozilla-esr38/',
 }
 
 REPO_SHORT_NAMES = {
-'mozilla-central':  'moz',
-'dom-inspector':    'dom',
-'ldap-sdks':        'ldap',
+    'mozilla-central':  'moz',
+    'dom-inspector':    'dom',
+    'ldap-sdks':        'ldap',
 }
 
+
 def get_DEFAULT_tag(index):
-  if index in DEFAULTS:
-    return DEFAULTS[index]
-  else:
-    return DEFAULTS['REV']
+    if index in DEFAULTS:
+        return DEFAULTS[index]
+    else:
+        return DEFAULTS['REV']
 
 # The set of defaults below relate to the current switching mechanism between
 # trunk or branches and back again if it is required.
@@ -69,8 +65,8 @@ SWITCH_MOZILLA_REPO_BACKUP_LOCATION = ".mozilla-1.9.1"
 # This is the potential location for a repository from the last time we
 # switched. Can be blank for no effect.
 SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION = ".mozilla-trunk"
-# This should be the same as DEFAULTS['MOZILLA_REPO'] but using %s instead of http
-# for the scheme.
+# This should be the same as DEFAULTS['MOZILLA_REPO'] but using %s instead
+# of http for the scheme.
 SWITCH_MOZILLA_REPO_REPLACE = '%s://hg.mozilla.org/mozilla-central/'
 SWITCH_MOZILLA_BASE_REV = "GECKO_1_9_1_BASE"
 
@@ -78,15 +74,14 @@ import sys
 # Test Python Version. 2.4 required for `import subprocess`
 pyver = sys.version_info
 if pyver[0] <= 1 or (pyver[0] == 2 and pyver[1] < 4):
-  sys.exit("ERROR: Python 2.4 or newer required")
+    sys.exit("ERROR: Python 2.4 or newer required")
 elif pyver[0] >= 3:
-  # Python series 3 will syntax error here, Hack needed per Bug 601649c#8
-  print "ERROR: Python series 3 is not supported, use series 2 >= 2.4"
-  sys.exit() # Do an explicit sys.exit for code clarity.
+    # Python series 3 will syntax error here, Hack needed per Bug 601649c#8
+    print "ERROR: Python series 3 is not supported, use series 2 >= 2.4"
+    sys.exit()  # Do an explicit sys.exit for code clarity.
 del pyver
 
 import os
-import datetime
 from optparse import OptionParser, OptionValueError
 import urllib2
 import re
@@ -101,6 +96,7 @@ try:
     from subprocess import check_call
 except ImportError:
     import subprocess
+
     def check_call(*popenargs, **kwargs):
         retcode = subprocess.call(*popenargs, **kwargs)
         if retcode:
@@ -113,6 +109,7 @@ try:
     from subprocess import check_output
 except ImportError:
     import subprocess
+
     def check_output(*popenargs, **kwargs):
         if 'stdout' in kwargs:
             raise ValueError('stdout argument not allowed, it will be overridden.')
@@ -126,46 +123,50 @@ except ImportError:
             raise CalledProcessError(retcode, cmd, output=output)
         return output
 
+
 def check_call_output(cmd, *args, **kwargs):
     return check_output(cmd, *args, **kwargs)
 
+
 def check_call_noisy(cmd, retryMax=0, *args, **kwargs):
-  """Wrapper around execute_check_call() to allow retries before failing.
+    """Wrapper around execute_check_call() to allow retries before failing.
 
-  |cmd|, is the command to try and execute.
-  |retryMax|, is the maximum number of retries to attempt, 0 by default.
-  """
+    |cmd|, is the command to try and execute.
+    |retryMax|, is the maximum number of retries to attempt, 0 by default.
+    """
 
-  def execute_check_call(cmd, *args, **kwargs):
-    print "Executing command:", cmd
-    check_call(cmd, *args, **kwargs)
+    def execute_check_call(cmd, *args, **kwargs):
+        print "Executing command:", cmd
+        check_call(cmd, *args, **kwargs)
 
-  # By default (= no retries), simply pass the call on.
-  if retryMax == 0:
-    execute_check_call(cmd, *args, **kwargs)
-    return
+    # By default (= no retries), simply pass the call on.
+    if retryMax == 0:
+        execute_check_call(cmd, *args, **kwargs)
+        return
+
+    # Loop (1 + retryMax) times, at most.
+    for r in range(0, 1 + retryMax):
+        # Add a retry header, not for initial call.
+        if r != 0:
+            print >> sys.stderr, "Retrying previous command: %d of %d time(s)" % (r, retryMax)
+        try:
+            # (Re-)Try the call.
+            execute_check_call(cmd, *args, **kwargs)
+            # If the call succeeded then no more retries.
+            break
+        except:
+            # Print the exception without its traceback.
+            # This traceback starts in the try block, which should be low value.
+            print >> sys.stderr, "The exception was:"
+            sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], None)
+            # Add a blank line.
+            print >> sys.stderr
+        else:
+            # Raise our own exception.
+            # This traceback starts at (= reports) the initial caller line.
+            raise Exception("Command '%s' failed %d time(s). Giving up." %
+                            (cmd, retryMax + 1))
 
-  # Loop (1 + retryMax) times, at most.
-  for r in range(0, 1 + retryMax):
-    # Add a retry header, not for initial call.
-    if r != 0:
-      print >> sys.stderr, "Retrying previous command: %d of %d time(s)" % (r, retryMax)
-    try:
-      # (Re-)Try the call.
-      execute_check_call(cmd, *args, **kwargs)
-      # If the call succeeded then no more retries.
-      break
-    except:
-      # Print the exception without its traceback.
-      # This traceback starts in the try block, which should be low value.
-      print >> sys.stderr, "The exception was:"
-      sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], None)
-      # Add a blank line.
-      print >> sys.stderr
-  else:
-    # Raise our own exception.
-    # This traceback starts at (= reports) the initial caller line.
-    raise Exception("Command '%s' failed %d time(s). Giving up." % (cmd, retryMax + 1))
 
 def repo_config():
     """Create/Update TREE_STATE_FILE as needed.
@@ -201,7 +202,8 @@ def repo_config():
     if not os.path.exists(TREE_STATE_FILE) or config_version < CURRENT_TREESTATE_VERSION:
         if not config.has_section('treestate'):
             config.add_section('treestate')
-        config.set('treestate', 'src_update_version', CURRENT_TREESTATE_VERSION)
+        config.set('treestate', 'src_update_version',
+                   CURRENT_TREESTATE_VERSION)
 
         # Write this file out
         f = open(TREE_STATE_FILE, 'w')
@@ -210,6 +212,7 @@ def repo_config():
         finally:
             f.close()
 
+
 def switch_mozilla_repo():
     """If the mozilla/ repo matches SWITCH_MOZILLA_REPO_REGEXP then:
     1) Backup (unused anymore) checkout of mozilla/.
@@ -224,9 +227,10 @@ def switch_mozilla_repo():
     mozilla_path = os.path.join(topsrcdir, 'mozilla')
     # Do nothing if there is no Mozilla directory.
     if not os.path.exists(mozilla_path):
-      return
+        return
 
-    import ConfigParser, re
+    import ConfigParser
+    import re
     config = ConfigParser.ConfigParser()
     config_path = os.path.join(mozilla_path, '.hg', 'hgrc')
     config.read([config_path])
@@ -242,14 +246,14 @@ def switch_mozilla_repo():
         return
 
     config.set('paths', 'default',
-               SWITCH_MOZILLA_REPO_REPLACE % match.group(1) )
+               SWITCH_MOZILLA_REPO_REPLACE % match.group(1))
 
     if config.has_option('paths', 'default-push'):
-      match = moz_old_regex.match(config.get('paths', 'default-push'))
-      # Do not update this property if not pushing to Mozilla trunk.
-      if match:
-        config.set('paths', 'default-push',
-                   SWITCH_MOZILLA_REPO_REPLACE % match.group(1) )
+        match = moz_old_regex.match(config.get('paths', 'default-push'))
+        # Do not update this property if not pushing to Mozilla trunk.
+        if match:
+            config.set('paths', 'default-push',
+                       SWITCH_MOZILLA_REPO_REPLACE % match.group(1))
 
     hgcloneopts = []
     if options.hgcloneopts:
@@ -259,7 +263,8 @@ def switch_mozilla_repo():
     if options.hgopts:
         hgopts = options.hgopts.split()
 
-    backup_mozilla_path = os.path.join(topsrcdir, SWITCH_MOZILLA_REPO_BACKUP_LOCATION)
+    backup_mozilla_path = os.path.join(topsrcdir,
+                                       SWITCH_MOZILLA_REPO_BACKUP_LOCATION)
     print "Moving mozilla to " + SWITCH_MOZILLA_REPO_BACKUP_LOCATION + "..."
     try:
         os.rename(mozilla_path, backup_mozilla_path)
@@ -269,7 +274,8 @@ def switch_mozilla_repo():
         sys.exit("Error: Mozilla directory renaming failed!")
 
     # Does the user have a pre-existing backup repository?
-    old_backup_repository = os.path.join(topsrcdir, SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION)
+    old_backup_repository = os.path.join(topsrcdir,
+                                         SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION)
     if SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION != "" and \
             os.path.exists(old_backup_repository):
         # Yes, so let's use that
@@ -285,16 +291,18 @@ def switch_mozilla_repo():
         return
 
     # Locally clone common repository history.
-    check_call_noisy([options.hg, 'clone', '-r', SWITCH_MOZILLA_BASE_REV] + hgcloneopts + hgopts + [backup_mozilla_path, mozilla_path],
+    check_call_noisy([options.hg, 'clone', '-r', SWITCH_MOZILLA_BASE_REV] +
+                     hgcloneopts + hgopts + [backup_mozilla_path, mozilla_path],
                      retryMax=options.retries)
 
     # Rewrite hgrc for new local mozilla repo based on pre-existing hgrc
     # but with new values
     f = open(os.path.join(topsrcdir, 'mozilla', '.hg', 'hgrc'), 'w')
     try:
-      config.write(f)
+        config.write(f)
     finally:
-      f.close()
+        f.close()
+
 
 def backup_cvs_extension(extensionName, extensionDir, extensionPath):
     """Backup (obsolete) cvs checkout of extensionName.
@@ -313,7 +321,8 @@ def backup_cvs_extension(extensionName, extensionDir, extensionPath):
         sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], None)
         sys.exit("Error: %s directory renaming failed!" % extensionName)
 
-def do_hg_pull(dir, repository, hg, rev, hgtool=None):
+
+def do_hg_pull(dir, repository, hg, rev, hgtool=None, hgtool1=None):
     """Clone if the dir doesn't exist, pull if it does.
     """
 
@@ -327,66 +336,95 @@ def do_hg_pull(dir, repository, hg, rev, hgtool=None):
     if options.hgopts:
         hgopts = options.hgopts.split()
 
+    if hgtool or hgtool1:
+        if not os.path.exists(hgtool):
+            hgtool = hgtool1
+
     if hgtool:
         hgtoolcmd = hgtool.split()
-        # We need to strip the trailing slash from the repository url so that the hg tool gets a
-        # url that's consistent with the rest of the build automation.
-        check_call_noisy(['python'] + hgtoolcmd + [repository.rstrip('/'), fulldir], retryMax=options.retries)
-    elif not os.path.exists(fulldir):
-        fulldir = os.path.join(topsrcdir, dir)
-        check_call_noisy([hg, 'clone'] + hgcloneopts + hgopts + [repository, fulldir],
+        # We need to strip the trailing slash from the repository url so that
+        # the hg tool gets a url that's consistent with the rest of the build
+        # automation.
+        repo = repository.rstrip('/')
+
+        # Make sure to unset the PROPERTIES_FILE variable, otherwise hgtool
+        # might use this for the revisions and branches.
+        hgtoolenv = os.environ.copy()
+        if 'PROPERTIES_FILE' in hgtoolenv:
+            del hgtoolenv['PROPERTIES_FILE']
+
+        # Set up the tool options, most importantly the revision
+        hgtoolopts = ['-r', rev]
+        if options.verbose:
+            hgtoolopts.append('-v')
+
+        check_call_noisy(['python'] + hgtoolcmd +
+                         hgtoolopts +
+                         [repo, fulldir],
+                         env=hgtoolenv,
                          retryMax=options.retries)
     else:
-        cmd = [hg, 'pull', '-R', fulldir] + hgopts
-        if repository is not None:
-            cmd.append(repository)
-        check_call_noisy(cmd, retryMax=options.retries)
+        if not os.path.exists(fulldir):
+            fulldir = os.path.join(topsrcdir, dir)
+            check_call_noisy([hg, 'clone'] + hgcloneopts + hgopts +
+                             [repository, fulldir],
+                             retryMax=options.retries)
+        else:
+            cmd = [hg, 'pull', '-R', fulldir] + hgopts
+            if repository is not None:
+                cmd.append(repository)
+            check_call_noisy(cmd, retryMax=options.retries)
 
-    # update to specific revision
-    cmd = [hg, 'update', '-r', rev, '-R', fulldir ] + hgopts
-    if options.verbose:
-        cmd.append('-v')
-    # Explicitly never retry 'hg update': otherwise any merge failures are ignored.
-    # This command is local: a failure can't be caused by a network error.
-    check_call_noisy(cmd, retryMax=0)
+        # update to specific revision
+        cmd = [hg, 'update', '-r', rev, '-R', fulldir] + hgopts
+        if options.verbose:
+            cmd.append('-v')
+        # Explicitly never retry 'hg update': otherwise any merge failures are
+        # ignored.
+        # This command is local: a failure can't be caused by a network error.
+        check_call_noisy(cmd, retryMax=0)
 
     check_call([hg, 'parent', '-R', fulldir,
                 '--template=Updated to revision {node}.\n'])
 
     if options.tinderbox_print and dir != '.':
         got_rev = check_call_output([hg, 'parent', '-R', fulldir,
-                    '--template={node|short}'])
+                                     '--template={node|short}'])
 
-        url = check_call_output([hg, 'paths', 'default', '-R', fulldir ]).rstrip().rstrip('/')
+        url = check_call_output([hg, 'paths', 'default', '-R', fulldir]).rstrip().rstrip('/')
         repo_name = url.split('/')[-1]
         repo_short_name = REPO_SHORT_NAMES.get(repo_name, repo_name)
 
-        print "TinderboxPrint:<a href=%s/rev/%s title='Built from %s revision %s'>%s:%s</a>"  % ( url, got_rev, repo_name, got_rev, repo_short_name, got_rev)
+        print "TinderboxPrint:<a href=%s/rev/%s title='Built from %s revision %s'>%s:%s</a>" % (url, got_rev, repo_name, got_rev, repo_short_name, got_rev)
 
 
 def check_retries_option(option, opt_str, value, parser):
-  if value < 0:
-    raise OptionValueError("%s option value needs to be positive (not '%d')" % (opt_str, value))
-  setattr(parser.values, option.dest, value)
+    if value < 0:
+        raise OptionValueError("%s option value needs to be positive (not '%d')" %
+                               (opt_str, value))
+    setattr(parser.values, option.dest, value)
+
 
 def do_apply_patch(hg, patch, repo):
-    check_call_noisy([hg, 
-        'import', 
-        '-R', repo,
-        '-m', "local patch from %s" % patch,
-        '--no-commit', '--force',
-        patch,
-        ], 
-        retryMax=0)
+    check_call_noisy([hg,
+                      'import',
+                      '-R', repo,
+                      '-m', "local patch from %s" % patch,
+                      '--no-commit', '--force',
+                      patch,
+                      ],
+                     retryMax=0
+                     )
     return
 
 import glob
+
+
 def do_apply_patches(topsrcdir, hg):
     prefix_map = {
         'mozilla': 'mozilla',
         'chatzilla': os.path.join('mozilla', 'extensions', 'irc'),
         'inspector': os.path.join('mozilla', 'extensions', 'inspector'),
-        'venkman':   os.path.join('mozilla', 'extensions', 'venkman'),
         'ldap':      os.path.join('ldap', 'sdks'),
     }
 
@@ -397,7 +435,7 @@ def do_apply_patches(topsrcdir, hg):
         for file in files:
             patch = os.path.join(topsrcdir, file)
             if os.path.exists(patch):
-               do_apply_patch(hg, patch, prefix_dir)
+                do_apply_patch(hg, patch, prefix_dir)
 
 o = OptionParser(usage="%prog [options] checkout")
 o.add_option("-m", "--comm-repo", dest="comm_repo",
@@ -443,25 +481,22 @@ o.add_option("--ldap-rev", dest="ldap_rev",
              default=None,
              help="Revision of LDAP repository to update to. Default: \"" + get_DEFAULT_tag('LDAPSDKS_REV') + "\"")
 
-o.add_option("--chatzilla-repo", dest = "chatzilla_repo",
-             default = None,
-             help = "URL of ChatZilla repository to pull from (default: use hg default in mozilla/extensions/irc/.hg/hgrc; or if that file doesn't exist, use \"" + DEFAULTS['CHATZILLA_REPO'] + "\".)")
+o.add_option("--chatzilla-repo", dest="chatzilla_repo",
+             default=None,
+             help="URL of ChatZilla repository to pull from (default: use hg default in mozilla/extensions/irc/.hg/hgrc; or if that file doesn't exist, use \"" + DEFAULTS['CHATZILLA_REPO'] + "\".)")
 o.add_option("--skip-chatzilla", dest="skip_chatzilla",
              action="store_true", default=False,
              help="Skip pulling the ChatZilla repository.")
-o.add_option("--chatzilla-rev", dest = "chatzilla_rev",
-             default = None,
-             help = "Revision of ChatZilla repository to update to. Default: \"" + get_DEFAULT_tag('CHATZILLA_REV') + "\"")
+o.add_option("--chatzilla-rev", dest="chatzilla_rev",
+             default=None,
+             help="Revision of ChatZilla repository to update to. Default: \"" + get_DEFAULT_tag('CHATZILLA_REV') + "\"")
 
-o.add_option("--venkman-repo", dest = "venkman_repo",
-             default = None,
-             help = "URL of Venkman repository to pull from (default: use hg default in mozilla/extensions/venkman/.hg/hgrc; or if that file doesn't exist, use \"" + DEFAULTS['VENKMAN_REPO'] + "\".)")
 o.add_option("--skip-venkman", dest="skip_venkman",
              action="store_true", default=False,
-             help="Skip pulling the Venkman repository.")
-o.add_option("--venkman-rev", dest = "venkman_rev",
-             default = None,
-             help = "Revision of Venkman repository to update to. Default: \"" + get_DEFAULT_tag('VENKMAN_REV') + "\"")
+             help="Obsolete option, supported for backwards compat.")
+o.add_option("--venkman-rev", dest="venkman_rev",
+             default=None,
+             help="Obsolete option, supported for backwards compat.")
 
 o.add_option("--hg", dest="hg", default=os.environ.get('HG', 'hg'),
              help="The location of the hg binary")
@@ -471,6 +506,9 @@ o.add_option("-v", "--verbose", dest="verbose",
 o.add_option("--hgtool", dest="hgtool",
              default=None,
              help="Path to hgtool, if wanted")
+o.add_option("--hgtool1", dest="hgtool1",
+             default=None,
+             help="Alternative path to hgtool, if wanted")
 o.add_option("--hg-options", dest="hgopts",
              help="Pass arbitrary options to hg commands (i.e. --debug, --time)")
 o.add_option("--hg-clone-options", dest="hgcloneopts",
@@ -480,9 +518,9 @@ o.add_option("--retries", dest="retries", type="int", metavar="NUM",
              default=1, help="Number of times to retry a failed command before giving up. (default: 1)",
              action="callback", callback=check_retries_option)
 
-o.add_option("-r", "--rev", dest = "default_rev",
-             default = None,
-             help = "Revision of all repositories to update to, unless otherwise specified.")
+o.add_option("-r", "--rev", dest="default_rev",
+             default=None,
+             help="Revision of all repositories to update to, unless otherwise specified.")
 
 o.add_option("--apply-patches", dest="apply_patches",
              action="store_true", default=False,
@@ -492,6 +530,7 @@ o.add_option("--tinderbox-print", dest="tinderbox_print",
              action="store_true", default=False,
              help="Print repo revisions for Tinderbox")
 
+
 def fixup_comm_repo_options(options):
     """Check options.comm_repo value.
 
@@ -508,15 +547,21 @@ def fixup_comm_repo_options(options):
         print "Error: the -m option is required for the initial checkout!"
         sys.exit(2)
 
-    if options.comm_rev is None:
-        # If we weren't passed an explicit rev, try to find it from the
-        # environment. If that doesn't exist or is empty, grab the default.
-        envRev = os.environ.get("COMM_REV")
-        if envRev:
-            print "Using COMM_REV from environment (%s)" % envRev
-            options.comm_rev = envRev
+    # If COMM_REV is passed via environment, prefer that over the passed
+    # option so that automation can pass tagged versions while still defaulting
+    # to a certain revision via client.py-args.
+    envRev = os.environ.get("COMM_REV")
+    if envRev:
+        if options.comm_rev and options.comm_rev != envRev:
+            print "Warning: Preferring COMM_REV (%s) over passed revision (%s)" % (envRev, options.comm_rev)
         else:
-            options.comm_rev = get_DEFAULT_tag("COMM_REV")
+            print "Using COMM_REV from environment (%s)" % envRev
+        options.comm_rev = envRev
+
+    # If no version was set before, use the default comm revision.
+    if options.comm_rev is None:
+        options.comm_rev = get_DEFAULT_tag("COMM_REV")
+
 
 def fixup_mozilla_repo_options(options):
     """Handle special case: initial checkout of Mozilla.
@@ -528,25 +573,32 @@ def fixup_mozilla_repo_options(options):
             options.mozilla_repo = DEFAULTS['MOZILLA_REPO']
         else:
             # Fallback to using .hgrc as hgtool/share needs the repo
-            import ConfigParser, re
+            import ConfigParser
             config = ConfigParser.ConfigParser()
             config_path = os.path.join(topsrcdir, 'mozilla', '.hg', 'hgrc')
             config.read([config_path])
             if not config.has_option('paths', 'default'):
                 # Abort, not to get into a possibly inconsistent state.
-                sys.exit("Error: default path in %s is undefined!" % config_path)
+                sys.exit("Error: default path in %s is undefined!" %
+                         config_path)
 
             options.mozilla_repo = config.get('paths', 'default')
 
-    if options.mozilla_rev is None:
-        # If we weren't passed an explicit rev, try to find it from the
-        # environment. If that doesn't exist or is empty, grab the default.
-        envRev = os.environ.get("MOZILLA_REV")
-        if envRev:
-            print "Using MOZILLA_REV from environment (%s)" % envRev
-            options.mozilla_rev = envRev
+    # If MOZILLA_REV is passed via environment, prefer that over the passed
+    # option so that automation can pass tagged versions while still defaulting
+    # to a certain revision via client.py-args.
+    envRev = os.environ.get("MOZILLA_REV")
+    if envRev:
+        if options.mozilla_rev and options.mozilla_rev != envRev:
+            print "Warning: Preferring MOZILLA_REV (%s) over passed revision (%s)" % (envRev, options.mozilla_rev)
         else:
-            options.mozilla_rev = get_DEFAULT_tag("MOZILLA_REV")
+            print "Using MOZILLA_REV from environment (%s)" % envRev
+        options.mozilla_rev = envRev
+
+    # If no version was set before, use the default mozilla revision.
+    if options.mozilla_rev is None:
+        options.mozilla_rev = get_DEFAULT_tag("MOZILLA_REV")
+
 
 def fixup_chatzilla_repo_options(options):
     """Handle special case: initial hg checkout of Chatzilla.
@@ -565,6 +617,7 @@ def fixup_chatzilla_repo_options(options):
     if options.chatzilla_rev is None:
         options.chatzilla_rev = get_DEFAULT_tag("CHATZILLA_REV")
 
+
 def fixup_inspector_repo_options(options):
     """Handle special case: initial checkout of DOM Inspector.
 
@@ -574,28 +627,13 @@ def fixup_inspector_repo_options(options):
     # No cvs backup needed as DOM Inspector was part (and removed from)
     # Mozilla hg repository.
     if options.inspector_repo is None and \
-            not os.path.exists(os.path.join(topsrcdir, 'mozilla', 'extensions', 'inspector')):
+            not os.path.exists(os.path.join(topsrcdir, 'mozilla', 'extensions',
+                                            'inspector')):
         options.inspector_repo = DEFAULTS['INSPECTOR_REPO']
 
     if options.inspector_rev is None:
         options.inspector_rev = get_DEFAULT_tag("INSPECTOR_REV")
 
-def fixup_venkman_repo_options(options):
-    """Handle special case: initial hg checkout of Venkman.
-
-    See fixup_comm_repo_options().
-    backup_cvs_extension() is also called.
-    """
-
-    extensionPath = os.path.join(topsrcdir, 'mozilla', 'extensions', 'venkman')
-
-    backup_cvs_extension('Venkman', 'venkman', extensionPath)
-
-    if options.venkman_repo is None and not os.path.exists(extensionPath):
-        options.venkman_repo = DEFAULTS['VENKMAN_REPO']
-
-    if options.venkman_rev is None:
-        options.venkman_rev = get_DEFAULT_tag("VENKMAN_REV")
 
 def fixup_ldap_repo_options(options):
     """Handle special case: initial checkout of LDAP.
@@ -611,6 +649,7 @@ def fixup_ldap_repo_options(options):
     if options.ldap_rev is None:
         options.ldap_rev = get_DEFAULT_tag("LDAPSDKS_REV")
 
+
 def get_last_known_good_mozilla_rev():
     kg_url = "http://build.mozillamessaging.com/buildbot/production/known-good-revisions/mozilla-central.txt"
     try:
@@ -620,7 +659,8 @@ def get_last_known_good_mozilla_rev():
     if re.search(r'^[a-f0-9]+$', rev) and len(rev) == 12:
         return rev
     else:
-        sys.exit("Error: invalid contents fetched from %s: '%s'" % (kg_url, rev))
+        sys.exit("Error: invalid contents fetched from %s: '%s'" %
+                 (kg_url, rev))
 
 try:
     (options, (action,)) = o.parse_args()
@@ -629,14 +669,14 @@ except ValueError:
     sys.exit(2)
 
 if options.default_rev:
-  # We now wish to override all the DEFAULTS.
-  DEFAULTS['REV'] = options.default_rev
-  for index in ['CHATZILLA', 'INSPECTOR', 'VENKMAN', 'COMM', 'MOZILLA',
-                'LDAPSDKS']:
-    index += "_REV"
-    # Clear the rest from file-defaults
-    if index in DEFAULTS:
-      del DEFAULTS[index]
+    # We now wish to override all the DEFAULTS.
+    DEFAULTS['REV'] = options.default_rev
+    for index in ['CHATZILLA', 'INSPECTOR', 'COMM', 'MOZILLA',
+                  'LDAPSDKS']:
+        index += "_REV"
+        # Clear the rest from file-defaults
+        if index in DEFAULTS:
+            del DEFAULTS[index]
 
 if action in ('checkout', 'co'):
     # Update Comm repository configuration.
@@ -648,37 +688,36 @@ if action in ('checkout', 'co'):
 
     if not options.skip_mozilla:
         if options.known_good and options.mozilla_rev is None:
-            print "Fetching last known good mozilla revision" 
+            print "Fetching last known good mozilla revision"
             options.mozilla_rev = get_last_known_good_mozilla_rev()
             print "Setting mozilla_rev to '%s'" % options.mozilla_rev
-            
+
         fixup_mozilla_repo_options(options)
-        do_hg_pull('mozilla', options.mozilla_repo, options.hg, options.mozilla_rev, options.hgtool)
+        do_hg_pull('mozilla', options.mozilla_repo, options.hg,
+                   options.mozilla_rev, options.hgtool, options.hgtool1)
 
     # Check whether destination directory exists for these extensions.
-    if (not options.skip_chatzilla or not options.skip_inspector or \
-                not options.skip_venkman) and \
-            not os.path.exists(os.path.join(topsrcdir, 'mozilla', 'extensions')):
+    if (not options.skip_chatzilla or not options.skip_inspector) and \
+       not os.path.exists(os.path.join(topsrcdir, 'mozilla', 'extensions')):
         # Don't create the directory: Mozilla repository should provide it...
-        sys.exit("Error: mozilla/extensions directory does not exist;" + \
-                 " ChatZilla, DOM Inspector and/or Venkman cannot be checked out!")
+        sys.exit("Error: mozilla/extensions directory does not exist;" +
+                 " ChatZilla, and/or DOM Inspector cannot be checked out!")
 
     if not options.skip_chatzilla:
         fixup_chatzilla_repo_options(options)
-        do_hg_pull(os.path.join('mozilla', 'extensions', 'irc'), options.chatzilla_repo, options.hg, options.chatzilla_rev)
+        do_hg_pull(os.path.join('mozilla', 'extensions', 'irc'),
+                   options.chatzilla_repo, options.hg, options.chatzilla_rev)
 
     if not options.skip_inspector:
         fixup_inspector_repo_options(options)
-        do_hg_pull(os.path.join('mozilla', 'extensions', 'inspector'), options.inspector_repo, options.hg, options.inspector_rev)
+        do_hg_pull(os.path.join('mozilla', 'extensions', 'inspector'),
+                   options.inspector_repo, options.hg, options.inspector_rev)
 
     if not options.skip_ldap:
         fixup_ldap_repo_options(options)
-        do_hg_pull(os.path.join('ldap', 'sdks'), options.ldap_repo, options.hg, options.ldap_rev)
+        do_hg_pull(os.path.join('ldap', 'sdks'), options.ldap_repo, options.hg,
+                   options.ldap_rev)
 
-    if not options.skip_venkman:
-        fixup_venkman_repo_options(options)
-        do_hg_pull(os.path.join('mozilla', 'extensions', 'venkman'), options.venkman_repo, options.hg, options.venkman_rev)
-  
     if options.apply_patches:
         do_apply_patches(topsrcdir, options.hg)
 
