@@ -1,17 +1,20 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-
-"use strict";
+'use strict';
 
 const ERR_CONFLICT = 'Remaining conflicting property: ',
       ERR_REQUIRED = 'Missing required property: ';
 
-function assertSametrait(test, trait1, trait2) {
-  let names1 = Object.getOwnPropertyNames(trait1),
-      names2 = Object.getOwnPropertyNames(trait2);
+const getOwnIdentifiers = x => [...Object.getOwnPropertyNames(x),
+                                ...Object.getOwnPropertySymbols(x)];
 
-  test.assertEqual(
+
+function assertSametrait(assert, trait1, trait2) {
+  let names1 = getOwnIdentifiers(trait1),
+      names2 = getOwnIdentifiers(trait2);
+
+  assert.equal(
     names1.length,
     names2.length,
     'equal traits must have same amount of properties'
@@ -19,52 +22,54 @@ function assertSametrait(test, trait1, trait2) {
 
   for (let i = 0; i < names1.length; i++) {
     let name = names1[i];
-    test.assertNotEqual(
+    assert.notEqual(
       -1,
       names2.indexOf(name),
       'equal traits must contain same named properties: ' + name
     );
-    assertSameDescriptor(test, name, trait1[name], trait2[name]);
+    assertSameDescriptor(assert, name, trait1[name], trait2[name]);
   }
 }
 
-function assertSameDescriptor(test, name, desc1, desc2) {
+function assertSameDescriptor(assert, name, desc1, desc2) {
   if (desc1.conflict || desc2.conflict) {
-    test.assertEqual(
+    assert.equal(
       desc1.conflict,
       desc2.conflict,
       'if one of same descriptors has `conflict` another must have it: '
         + name
     );
-  } else if (desc1.required || desc2.required) {
-    test.assertEqual(
+  }
+  else if (desc1.required || desc2.required) {
+    assert.equal(
       desc1.required,
       desc2.required,
       'if one of same descriptors is has `required` another must have it: '
         + name
     );
-  } else {
-    test.assertEqual(
+  }
+  else {
+    assert.equal(
       desc1.get,
       desc2.get,
       'get must be the same on both descriptors: ' + name
     );
-    test.assertEqual(
+    assert.equal(
       desc1.set,
       desc2.set,
       'set must be the same on both descriptors: ' + name
     );
-    test.assertEqual(
+    assert.equal(
       desc1.value,
       desc2.value,
       'value must be the same on both descriptors: ' + name
     );
-    test.assertEqual(
+    assert.equal(
       desc1.enumerable,
       desc2.enumerable,
       'enumerable must be the same on both descriptors: ' + name
     );
-    test.assertEqual(
+    assert.equal(
       desc1.required,
       desc2.required,
       'value must be the same on both descriptors: ' + name
@@ -123,17 +128,17 @@ const { trait, compose, resolve, required, override, create } =
   require('sdk/deprecated/traits/core');
 
 
-exports['test:empty trait'] = function(test) {
+exports['test:empty trait'] = function(assert) {
   assertSametrait(
-    test,
+    assert,
     trait({}),
     {}
   );
 };
 
-exports['test:simple trait'] = function(test) {
+exports['test:simple trait'] = function(assert) {
   assertSametrait(
-    test,
+    assert,
     trait({
       a: 0,
       b: testMethod
@@ -145,9 +150,9 @@ exports['test:simple trait'] = function(test) {
   );
 };
 
-exports['test:simple trait with required prop'] = function(test) {
+exports['test:simple trait with required prop'] = function(assert) {
   assertSametrait(
-    test,
+    assert,
     trait({
       a: required,
       b: 1
@@ -159,25 +164,27 @@ exports['test:simple trait with required prop'] = function(test) {
   );
 };
 
-exports['test:ordering of trait properties is irrelevant'] = function(test) {
-  assertSametrait(test,
+exports['test:ordering of trait properties is irrelevant'] = function(assert) {
+  assertSametrait(
+    assert,
     trait({ a: 0, b: 1, c: required }),
     trait({ b: 1, c: required, a: 0 })
   );
 };
 
-exports['test:trait with accessor property'] = function(test) {
+exports['test:trait with accessor property'] = function(assert) {
   let record = { get a() {}, set a(v) {} };
   let get = Object.getOwnPropertyDescriptor(record,'a').get;
   let set = Object.getOwnPropertyDescriptor(record,'a').set;
-  assertSametrait(test,
+  assertSametrait(assert,
     trait(record),
     { a: Accessor(get, set ) }
   );
 };
 
-exports['test:simple composition'] = function(test) {
-  assertSametrait(test,
+exports['test:simple composition'] = function(assert) {
+  assertSametrait(
+    assert,
     compose(
       trait({ a: 0, b: 1 }),
       trait({ c: 2, d: testMethod })
@@ -191,8 +198,9 @@ exports['test:simple composition'] = function(test) {
   );
 };
 
-exports['test:composition with conflict'] = function(test) {
-  assertSametrait(test,
+exports['test:composition with conflict'] = function(assert) {
+  assertSametrait(
+    assert,
     compose(
       trait({ a: 0, b: 1 }),
       trait({ a: 2, c: testMethod })
@@ -206,8 +214,8 @@ exports['test:composition with conflict'] = function(test) {
 };
 
 exports['test:composition of identical props does not cause conflict'] =
-function(test) {
-  assertSametrait(test,
+function(assert) {
+  assertSametrait(assert,
     compose(
       trait({ a: 0, b: 1 }),
       trait({ a: 0, c: testMethod })
@@ -220,8 +228,8 @@ function(test) {
 };
 
 exports['test:composition with identical required props'] =
-function(test) {
-  assertSametrait(test,
+function(assert) {
+  assertSametrait(assert,
     compose(
       trait({ a: required, b: 1 }),
       trait({ a: required, c: testMethod })
@@ -234,8 +242,8 @@ function(test) {
   );
 };
 
-exports['test:composition satisfying a required prop'] = function (test) {
-  assertSametrait(test,
+exports['test:composition satisfying a required prop'] = function (assert) {
+  assertSametrait(assert,
     compose(
       trait({ a: required, b: 1 }),
       trait({ a: testMethod })
@@ -247,8 +255,8 @@ exports['test:composition satisfying a required prop'] = function (test) {
   );
 };
 
-exports['test:compose is neutral wrt conflicts'] = function (test) {
-  assertSametrait(test,
+exports['test:compose is neutral wrt conflicts'] = function (assert) {
+  assertSametrait(assert,
     compose(
       compose(
         trait({ a: 1 }),
@@ -263,8 +271,8 @@ exports['test:compose is neutral wrt conflicts'] = function (test) {
   );
 };
 
-exports['test:conflicting prop overrides required prop'] = function (test) {
-  assertSametrait(test,
+exports['test:conflicting prop overrides required prop'] = function (assert) {
+  assertSametrait(assert,
     compose(
       compose(
         trait({ a: 1 }),
@@ -278,8 +286,8 @@ exports['test:conflicting prop overrides required prop'] = function (test) {
   );
 };
 
-exports['test:compose is commutative'] = function (test) {
-  assertSametrait(test,
+exports['test:compose is commutative'] = function (assert) {
+  assertSametrait(assert,
     compose(
       trait({ a: 0, b: 1 }),
       trait({ c: 2, d: testMethod })
@@ -292,8 +300,8 @@ exports['test:compose is commutative'] = function (test) {
 };
 
 exports['test:compose is commutative, also for required/conflicting props'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     compose(
       trait({ a: 0, b: 1, c: 3, e: required }),
       trait({ c: 2, d: testMethod })
@@ -304,8 +312,8 @@ function (test) {
     )
   );
 };
-exports['test:compose is associative'] = function (test) {
-  assertSametrait(test,
+exports['test:compose is associative'] = function (assert) {
+  assertSametrait(assert,
     compose(
       trait({ a: 0, b: 1, c: 3, d: required }),
       compose(
@@ -324,8 +332,8 @@ exports['test:compose is associative'] = function (test) {
 };
 
 exports['test:diamond import of same prop does not generate conflict'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     compose(
       compose(
         trait({ b: 2 }),
@@ -347,8 +355,8 @@ function (test) {
 };
 
 exports['test:resolve with empty resolutions has no effect'] =
-function (test) {
-  assertSametrait(test, resolve({}, trait({
+function (assert) {
+  assertSametrait(assert, resolve({}, trait({
     a: 1,
     b: required,
     c: testMethod
@@ -359,8 +367,8 @@ function (test) {
   });
 };
 
-exports['test:resolve: renaming'] = function (test) {
-  assertSametrait(test,
+exports['test:resolve: renaming'] = function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'A', c: 'C' },
       trait({ a: 1, b: required, c: testMethod })
@@ -376,8 +384,8 @@ exports['test:resolve: renaming'] = function (test) {
 };
 
 exports['test:resolve: renaming to conflicting name causes conflict, order 1']
-= function (test) {
-  assertSametrait(test,
+= function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'b'},
       trait({ a: 1, b: 2 })
@@ -390,8 +398,8 @@ exports['test:resolve: renaming to conflicting name causes conflict, order 1']
 };
 
 exports['test:resolve: renaming to conflicting name causes conflict, order 2']
-= function (test) {
-  assertSametrait(test,
+= function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'b' },
       trait({ b: 2, a: 1 })
@@ -403,8 +411,8 @@ exports['test:resolve: renaming to conflicting name causes conflict, order 2']
   );
 };
 
-exports['test:resolve: simple exclusion'] = function (test) {
-  assertSametrait(test,
+exports['test:resolve: simple exclusion'] = function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: undefined },
       trait({ a: 1, b: 2 })
@@ -416,8 +424,8 @@ exports['test:resolve: simple exclusion'] = function (test) {
   );
 };
 
-exports['test:resolve: exclusion to "empty" trait'] = function (test) {
-  assertSametrait(test,
+exports['test:resolve: exclusion to "empty" trait'] = function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: undefined, b: undefined },
       trait({ a: 1, b: 2 })
@@ -430,8 +438,8 @@ exports['test:resolve: exclusion to "empty" trait'] = function (test) {
 };
 
 exports['test:resolve: exclusion and renaming of disjoint props'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: undefined, b: 'c' },
       trait({ a: 1, b: 2 })
@@ -445,8 +453,8 @@ function (test) {
 };
 
 exports['test:resolve: exclusion and renaming of overlapping props'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: undefined, b: 'a' },
       trait({ a: 1, b: 2 })
@@ -459,8 +467,8 @@ function (test) {
 };
 
 exports['test:resolve: renaming to a common alias causes conflict'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'c', b: 'c' },
       trait({ a: 1, b: 2 })
@@ -474,8 +482,8 @@ function (test) {
 };
 
 exports['test:resolve: renaming overrides required target'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { b: 'a' },
       trait({ a: required, b: 2 })
@@ -488,8 +496,8 @@ function (test) {
 };
 
 exports['test:resolve: renaming required properties has no effect'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { b: 'a' },
       trait({ a: 2, b: required })
@@ -502,8 +510,8 @@ function (test) {
 };
 
 exports['test:resolve: renaming of non-existent props has no effect'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'c', d: 'c' },
       trait({ a: 1, b: 2 })
@@ -517,8 +525,8 @@ function (test) {
 };
 
 exports['test:resolve: exclusion of non-existent props has no effect'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { b: undefined },
       trait({ a: 1 })
@@ -530,8 +538,8 @@ function (test) {
 };
 
 exports['test:resolve is neutral w.r.t. required properties'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'c', b: undefined },
       trait({ a: required, b: required, c: 'foo', d: 1 })
@@ -546,8 +554,8 @@ function (test) {
 };
 
 exports['test:resolve supports swapping of property names, ordering 1'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'b', b: 'a' },
       trait({ a: 1, b: 2 })
@@ -560,8 +568,8 @@ function (test) {
 };
 
 exports['test:resolve supports swapping of property names, ordering 2'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { b: 'a', a: 'b' },
       trait({ a: 1, b: 2 })
@@ -574,8 +582,8 @@ function (test) {
 };
 
 exports['test:resolve supports swapping of property names, ordering 3'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { b: 'a', a: 'b' },
       trait({ b: 2, a: 1 })
@@ -588,8 +596,8 @@ function (test) {
 };
 
 exports['test:resolve supports swapping of property names, ordering 4'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     resolve(
       { a: 'b', b: 'a' },
       trait({ b: 2, a: 1 })
@@ -601,8 +609,8 @@ function (test) {
   );
 };
 
-exports['test:override of mutually exclusive traits'] = function (test) {
-  assertSametrait(test,
+exports['test:override of mutually exclusive traits'] = function (assert) {
+  assertSametrait(assert,
     override(
       trait({ a: 1, b: 2 }),
       trait({ c: 3, d: testMethod })
@@ -617,8 +625,8 @@ exports['test:override of mutually exclusive traits'] = function (test) {
 };
 
 exports['test:override of mutually exclusive traits is compose'] =
-function (test) {
-  assertSametrait(test,
+function (assert) {
+  assertSametrait(assert,
     override(
       trait({ a: 1, b: 2 }),
       trait({ c: 3, d: testMethod })
@@ -630,8 +638,8 @@ function (test) {
   );
 };
 
-exports['test:override of overlapping traits'] = function (test) {
-  assertSametrait(test,
+exports['test:override of overlapping traits'] = function (assert) {
+  assertSametrait(assert,
     override(
       trait({ a: 1, b: 2 }),
       trait({ a: 3, c: testMethod })
@@ -644,8 +652,8 @@ exports['test:override of overlapping traits'] = function (test) {
   );
 };
 
-exports['test:three-way override of overlapping traits'] = function (test) {
-  assertSametrait(test,
+exports['test:three-way override of overlapping traits'] = function (assert) {
+  assertSametrait(assert,
     override(
       trait({ a: 1, b: 2 }),
       trait({ b: 4, c: 3 }),
@@ -660,8 +668,8 @@ exports['test:three-way override of overlapping traits'] = function (test) {
   );
 };
 
-exports['test:override replaces required properties'] = function (test) {
-  assertSametrait(test,
+exports['test:override replaces required properties'] = function (assert) {
+  assertSametrait(assert,
     override(
       trait({ a: required, b: 2 }),
       trait({ a: 1, c: testMethod })
@@ -674,8 +682,8 @@ exports['test:override replaces required properties'] = function (test) {
   );
 };
 
-exports['test:override is not commutative'] = function (test) {
-  assertSametrait(test,
+exports['test:override is not commutative'] = function (assert) {
+  assertSametrait(assert,
     override(
       trait({ a: 1, b: 2 }),
       trait({ a: 3, c: 4 })
@@ -687,7 +695,7 @@ exports['test:override is not commutative'] = function (test) {
     }
   );
 
-  assertSametrait(test,
+  assertSametrait(assert,
     override(
       trait({ a: 3, c: 4 }),
       trait({ a: 1, b: 2 })
@@ -700,8 +708,8 @@ exports['test:override is not commutative'] = function (test) {
   );
 };
 
-exports['test:override is associative'] = function (test) {
-  assertSametrait(test,
+exports['test:override is associative'] = function (assert) {
+  assertSametrait(assert,
     override(
       override(
         trait({ a: 1, b: 2 }),
@@ -719,29 +727,29 @@ exports['test:override is associative'] = function (test) {
   );
 };
 
-exports['test:create simple'] = function(test) {
+exports['test:create simple'] = function(assert) {
   let o1 = create(
     Object.prototype,
     trait({ a: 1, b: function() { return this.a; } })
   );
 
-  test.assertEqual(
+  assert.equal(
     Object.prototype,
     Object.getPrototypeOf(o1),
     'o1 prototype'
   );
-  test.assertEqual(1, o1.a, 'o1.a');
-  test.assertEqual(1, o1.b(), 'o1.b()');
-  test.assertEqual(
+  assert.equal(1, o1.a, 'o1.a');
+  assert.equal(1, o1.b(), 'o1.b()');
+  assert.equal(
     2,
-    Object.getOwnPropertyNames(o1).length,
+    getOwnIdentifiers(o1).length,
     'Object.keys(o1).length === 2'
   );
 };
 
-exports['test:create with Array.prototype'] = function(test) {
+exports['test:create with Array.prototype'] = function(assert) {
   let o2 = create(Array.prototype, trait({}));
-  test.assertEqual(
+  assert.equal(
     Array.prototype,
     Object.getPrototypeOf(o2),
     "o2 prototype"
@@ -749,12 +757,13 @@ exports['test:create with Array.prototype'] = function(test) {
 };
 
 exports['test:exception for incomplete required properties'] =
-function(test) {
+function(assert) {
   try {
     create(Object.prototype, trait({ foo: required }));
-    test.fail('expected create to complain about missing required props');
-  } catch(e) {
-    test.assertEqual(
+    assert.fail('expected create to complain about missing required props');
+  }
+  catch(e) {
+    assert.equal(
       'Error: Missing required property: foo',
       e.toString(),
       'required prop error'
@@ -762,12 +771,13 @@ function(test) {
   }
 };
 
-exports['test:exception for unresolved conflicts'] = function(test) {
+exports['test:exception for unresolved conflicts'] = function(assert) {
   try {
     create({}, compose(trait({ a: 0 }), trait({ a: 1 })));
-    test.fail('expected create to complain about unresolved conflicts');
-  } catch(e) {
-    test.assertEqual(
+    assert.fail('expected create to complain about unresolved conflicts');
+  }
+  catch(e) {
+    assert.equal(
       'Error: Remaining conflicting property: a',
       e.toString(),
       'conflicting prop error'
@@ -776,49 +786,53 @@ exports['test:exception for unresolved conflicts'] = function(test) {
 };
 
 exports['test:verify that required properties are present but undefined'] =
-function(test) {
+function(assert) {
   try {
     let o4 = Object.create(Object.prototype, trait({ foo: required }));
-    test.assertEqual(true, 'foo' in o4, 'required property present');
+    assert.equal(true, 'foo' in o4, 'required property present');
     try {
       let foo = o4.foo;
-      test.fail('access to required property must throw');
-    } catch(e) {
-      test.assertEqual(
+      assert.fail('access to required property must throw');
+    }
+    catch(e) {
+      assert.equal(
         'Error: Missing required property: foo',
         e.toString(),
         'required prop error'
       )
     }
-  } catch(e) {
-    test.fail('did not expect create to complain about required props');
+  }
+  catch(e) {
+    assert.fail('did not expect create to complain about required props');
   }
 };
 
 exports['test:verify that conflicting properties are present'] =
-function(test) {
+function(assert) {
   try {
     let o5 = Object.create(
       Object.prototype,
       compose(trait({ a: 0 }), trait({ a: 1 }))
     );
-    test.assertEqual(true, 'a' in o5, 'conflicting property present');
+    assert.equal(true, 'a' in o5, 'conflicting property present');
     try {
       let a = o5.a; // accessors or data prop
-      test.fail('expected conflicting prop to cause exception');
-    } catch (e) {
-      test.assertEqual(
+      assert.fail('expected conflicting prop to cause exception');
+    }
+    catch (e) {
+      assert.equal(
         'Error: Remaining conflicting property: a',
         e.toString(),
         'conflicting prop access error'
       );
     }
-  } catch(e) {
-    test.fail('did not expect create to complain about conflicting props');
+  }
+  catch(e) {
+    assert.fail('did not expect create to complain about conflicting props');
   }
 };
 
-exports['test diamond with conflicts'] = function(test) {
+exports['test diamond with conflicts'] = function(assert) {
   function makeT1(x) trait({ m: function() { return x; } })
   function makeT2(x) compose(trait({ t2: 'foo' }), makeT1(x))
   function makeT3(x) compose(trait({ t3: 'bar' }), makeT1(x))
@@ -826,9 +840,10 @@ exports['test diamond with conflicts'] = function(test) {
   let T4 = compose(makeT2(5), makeT3(5));
   try {
     let o = create(Object.prototype, T4);
-    test.fail('expected diamond prop to cause exception');
-  } catch(e) {
-    test.assertEqual(
+    assert.fail('expected diamond prop to cause exception');
+  }
+  catch(e) {
+    assert.equal(
       'Error: Remaining conflicting property: m',
       e.toString(),
       'diamond prop conflict'
@@ -836,3 +851,4 @@ exports['test diamond with conflicts'] = function(test) {
   }
 };
 
+require('sdk/test').run(exports);
