@@ -4,9 +4,7 @@ function test() {
   var tab1 = gBrowser.addTab("data:text/plain;charset=utf-8,foo");
   gBrowser.pinTab(tab1);
 
-  tab1.linkedBrowser.addEventListener("load", function () {
-    tab1.linkedBrowser.removeEventListener("load", arguments.callee, true);
-
+  promiseBrowserLoaded(tab1.linkedBrowser).then(() => {
     var tab2 = gBrowser.addTab();
     gBrowser.pinTab(tab2);
 
@@ -19,5 +17,5 @@ function test() {
     gBrowser.removeTab(tab1);
     gBrowser.removeTab(tab2);
     finish();
-  }, true);
+  });
 }
