@@ -31,5 +31,5 @@ function test() {
         },
       },
     ]);
-  }).then(finish);
+  }).then(finish, helpers.handleError);
 }
