@@ -6,13 +6,19 @@ const Cu = Components.utils;
 let {devtools} = Cu.import("resource://gre/modules/devtools/Loader.jsm", {});
 let TargetFactory = devtools.TargetFactory;
 let {console} = Cu.import("resource://gre/modules/devtools/Console.jsm", {});
-let promise = devtools.require("devtools/toolkit/deprecated-sync-thenables");
+let promise = devtools.require("resource://gre/modules/Promise.jsm").Promise;
 let {getInplaceEditorForSpan: inplaceEditor} = devtools.require("devtools/shared/inplace-editor");
+let clipboard = devtools.require("sdk/clipboard");
 
 // All test are asynchronous
 waitForExplicitFinish();
 
-//Services.prefs.setBoolPref("devtools.dump.emit", true);
+// If a test times out we want to see the complete log and not just the last few
+// lines.
+SimpleTest.requestCompleteLog();
+
+// Uncomment this pref to dump all devtools emitted events to the console.
+// Services.prefs.setBoolPref("devtools.dump.emit", true);
 
 // Set the testing flag on gDevTools and reset it when the test ends
 gDevTools.testing = true;
@@ -25,29 +31,22 @@ registerCleanupFunction(() => {
   Services.prefs.clearUserPref("devtools.inspector.activeSidebar");
   Services.prefs.clearUserPref("devtools.dump.emit");
   Services.prefs.clearUserPref("devtools.markup.pagesize");
+  Services.prefs.clearUserPref("dom.webcomponents.enabled");
+  Services.prefs.clearUserPref("devtools.inspector.showAllAnonymousContent");
 });
 
 // Auto close the toolbox and close the test tabs when the test ends
-registerCleanupFunction(() => {
-  try {
-    let target = TargetFactory.forTab(gBrowser.selectedTab);
-    gDevTools.closeToolbox(target);
-  } catch (ex) {
-    dump(ex);
-  }
+registerCleanupFunction(function*() {
+  let target = TargetFactory.forTab(gBrowser.selectedTab);
+  yield gDevTools.closeToolbox(target);
+
   while (gBrowser.tabs.length > 1) {
     gBrowser.removeCurrentTab();
   }
 });
 
 const TEST_URL_ROOT = "http://mochi.test:8888/browser/browser/devtools/markupview/test/";
-
-/**
- * Define an async test based on a generator function
- */
-function asyncTest(generator) {
-  return () => Task.spawn(generator).then(null, ok.bind(null, false)).then(finish);
-}
+const CHROME_BASE = "chrome://mochitests/content/browser/browser/devtools/markupview/test/";
 
 /**
  * Add a new test tab in the browser and load the given url.
@@ -58,15 +57,19 @@ function addTab(url) {
   info("Adding a new tab with URL: '" + url + "'");
   let def = promise.defer();
 
-  let tab = gBrowser.selectedTab = gBrowser.addTab();
-  gBrowser.selectedBrowser.addEventListener("load", function onload() {
-    gBrowser.selectedBrowser.removeEventListener("load", onload, true);
+  // Bug 921935 should bring waitForFocus() support to e10s, which would
+  // probably cover the case of the test losing focus when the page is loading.
+  // For now, we just make sure the window is focused.
+  window.focus();
+
+  let tab = window.gBrowser.selectedTab = window.gBrowser.addTab(url);
+  let linkedBrowser = tab.linkedBrowser;
+
+  linkedBrowser.addEventListener("load", function onload() {
+    linkedBrowser.removeEventListener("load", onload, true);
     info("URL '" + url + "' loading complete");
-    waitForFocus(() => {
-      def.resolve(tab);
-    }, content);
+    def.resolve(tab);
   }, true);
-  content.location = url;
 
   return def.promise;
 }
@@ -124,7 +127,8 @@ function openInspector() {
  * Simple DOM node accesor function that takes either a node or a string css
  * selector as argument and returns the corresponding node
  * @param {String|DOMNode} nodeOrSelector
- * @return {DOMNode}
+ * @return {DOMNode|CPOW} Note that in e10s mode a CPOW object is returned which
+ * doesn't implement *all* of the DOMNode's properties
  */
 function getNode(nodeOrSelector) {
   info("Getting the node for '" + nodeOrSelector + "'");
@@ -134,38 +138,83 @@ function getNode(nodeOrSelector) {
 }
 
 /**
- * Set the inspector's current selection to a node or to the first match of the
- * given css selector
+ * Get the NodeFront for a given css selector, via the protocol
+ * @param {String|NodeFront} selector
+ * @param {InspectorPanel} inspector The instance of InspectorPanel currently
+ * loaded in the toolbox
+ * @return {Promise} Resolves to the NodeFront instance
+ */
+function getNodeFront(selector, {walker}) {
+  if (selector._form) {
+    return selector;
+  }
+  return walker.querySelector(walker.rootNode, selector);
+}
+
+/**
+ * Highlight a node and set the inspector's current selection to the node or
+ * the first match of the given css selector.
  * @param {String|DOMNode} nodeOrSelector
- * @param {InspectorPanel} inspector The instance of InspectorPanel currently loaded in the toolbox
- * @param {String} reason Defaults to "test" which instructs the inspector not to highlight the node upon selection
+ * @param {InspectorPanel} inspector
+ *        The instance of InspectorPanel currently loaded in the toolbox
  * @return a promise that resolves when the inspector is updated with the new
  * node
  */
-function selectNode(nodeOrSelector, inspector, reason="test") {
-  info("Selecting the node for '" + nodeOrSelector + "'");
+function selectAndHighlightNode(nodeOrSelector, inspector) {
+  info("Highlighting and selecting the node " + nodeOrSelector);
+
   let node = getNode(nodeOrSelector);
-  let updated = inspector.once("inspector-updated");
-  inspector.selection.setNode(node, reason);
+  let updated = inspector.toolbox.once("highlighter-ready");
+  inspector.selection.setNode(node, "test-highlight");
   return updated;
 }
 
+/**
+ * Set the inspector's current selection to the first match of the given css
+ * selector
+ * @param {String|NodeFront} selector
+ * @param {InspectorPanel} inspector The instance of InspectorPanel currently
+ * loaded in the toolbox
+ * @param {String} reason Defaults to "test" which instructs the inspector not
+ * to highlight the node upon selection
+ * @return {Promise} Resolves when the inspector is updated with the new node
+ */
+let selectNode = Task.async(function*(selector, inspector, reason="test") {
+  info("Selecting the node for '" + selector + "'");
+  let nodeFront = yield getNodeFront(selector, inspector);
+  let updated = inspector.once("inspector-updated");
+  inspector.selection.setNodeFront(nodeFront, reason);
+  yield updated;
+});
+
 /**
  * Get the MarkupContainer object instance that corresponds to the given
- * HTML node
- * @param {DOMNode|String} nodeOrSelector The DOM node for which the
- * container is required
+ * NodeFront
+ * @param {NodeFront} nodeFront
  * @param {InspectorPanel} inspector The instance of InspectorPanel currently
  * loaded in the toolbox
  * @return {MarkupContainer}
  */
-function getContainerForRawNode(nodeOrSelector, {markup}) {
-  let front = markup.walker.frontForRawNode(getNode(nodeOrSelector));
-  let container = markup.getContainer(front);
-  info("Markup-container object for " + nodeOrSelector + " " + container);
-  return container;
+function getContainerForNodeFront(nodeFront, {markup}) {
+  return markup.getContainer(nodeFront);
 }
 
+/**
+ * Get the MarkupContainer object instance that corresponds to the given
+ * selector
+ * @param {String|NodeFront} selector
+ * @param {InspectorPanel} inspector The instance of InspectorPanel currently
+ * loaded in the toolbox
+ * @return {MarkupContainer}
+ */
+let getContainerForSelector = Task.async(function*(selector, inspector) {
+  info("Getting the markup-container for node " + selector);
+  let nodeFront = yield getNodeFront(selector, inspector);
+  let container = getContainerForNodeFront(nodeFront, inspector);
+  info("Found markup-container " + container);
+  return container;
+});
+
 /**
  * Using the markupview's _waitForChildren function, wait for all queued
  * children updates to be handled.
@@ -183,40 +232,27 @@ function waitForChildrenUpdated({markup}) {
   return def.promise;
 }
 
-/**
- * Simulate a mouse-over on the markup-container (a line in the markup-view)
- * that corresponds to the node or selector passed.
- * @param {String|DOMNode} nodeOrSelector
- * @param {InspectorPanel} inspector The instance of InspectorPanel currently loaded in the toolbox
- * @return a promise that resolves when the container is hovered and the higlighter
- * is shown on the corresponding node
- */
-function hoverContainer(nodeOrSelector, inspector) {
-  info("Hovering over the markup-container for node " + nodeOrSelector);
-  let highlit = inspector.toolbox.once("node-highlight");
-  let container = getContainerForRawNode(getNode(nodeOrSelector), inspector);
-  EventUtils.synthesizeMouseAtCenter(container.tagLine, {type: "mousemove"},
-    inspector.markup.doc.defaultView);
-  return highlit;
-}
-
 /**
  * Simulate a click on the markup-container (a line in the markup-view)
- * that corresponds to the node or selector passed.
- * @param {String|DOMNode} nodeOrSelector
- * @param {InspectorPanel} inspector The instance of InspectorPanel currently loaded in the toolbox
- * @return a promise that resolves when the node has been selected.
+ * that corresponds to the selector passed.
+ * @param {String|NodeFront} selector
+ * @param {InspectorPanel} inspector The instance of InspectorPanel currently
+ * loaded in the toolbox
+ * @return {Promise} Resolves when the node has been selected.
  */
-function clickContainer(nodeOrSelector, inspector) {
-  info("Clicking on the markup-container for node " + nodeOrSelector);
-  let updated = inspector.once("inspector-updated");
-  let container = getContainerForRawNode(getNode(nodeOrSelector), inspector);
+let clickContainer = Task.async(function*(selector, inspector) {
+  info("Clicking on the markup-container for node " + selector);
+
+  let nodeFront = yield getNodeFront(selector, inspector);
+  let container = getContainerForNodeFront(nodeFront, inspector);
+
+  let updated = container.selected ? promise.resolve() : inspector.once("inspector-updated");
   EventUtils.synthesizeMouseAtCenter(container.tagLine, {type: "mousedown"},
     inspector.markup.doc.defaultView);
   EventUtils.synthesizeMouseAtCenter(container.tagLine, {type: "mouseup"},
     inspector.markup.doc.defaultView);
   return updated;
-}
+});
 
 /**
  * Checks if the highlighter is visible currently
@@ -228,25 +264,6 @@ function isHighlighterVisible() {
   return highlighter && !highlighter.hasAttribute("hidden");
 }
 
-/**
- * Simulate the mouse leaving the markup-view area
- * @param {InspectorPanel} inspector The instance of InspectorPanel currently loaded in the toolbox
- * @return a promise when done
- */
-function mouseLeaveMarkupView(inspector) {
-  info("Leaving the markup-view area");
-  let def = promise.defer();
-
-  // Find another element to mouseover over in order to leave the markup-view
-  let btn = inspector.toolbox.doc.querySelector(".toolbox-dock-button");
-
-  EventUtils.synthesizeMouseAtCenter(btn, {type: "mousemove"},
-    inspector.toolbox.doc.defaultView);
-  executeSoon(def.resolve);
-
-  return def.promise;
-}
-
 /**
  * Focus a given editable element, enter edit mode, set value, and commit
  * @param {DOMNode} field The element that gets editable after receiving focus
@@ -265,39 +282,39 @@ function setEditableFieldValue(field, value, inspector) {
 }
 
 /**
- * Focus the new-attribute inplace-editor field of the nodeOrSelector's markup
- * container, and enters the given text, then wait for it to be applied and the
- * for the node to mutates (when new attribute(s) is(are) created)
- * @param {DOMNode|String} nodeOrSelector The node or node selector to edit.
+ * Focus the new-attribute inplace-editor field of a node's markup container
+ * and enters the given text, then wait for it to be applied and the for the
+ * node to mutates (when new attribute(s) is(are) created)
+ * @param {String} selector The selector for the node to edit.
  * @param {String} text The new attribute text to be entered (e.g. "id='test'")
  * @param {InspectorPanel} inspector The instance of InspectorPanel currently
  * loaded in the toolbox
  * @return a promise that resolves when the node has mutated
  */
-function addNewAttributes(nodeOrSelector, text, inspector) {
-  info("Entering text '" + text + "' in node '" + nodeOrSelector + "''s new attribute field");
+let addNewAttributes = Task.async(function*(selector, text, inspector) {
+  info("Entering text '" + text + "' in node '" + selector + "''s new attribute field");
 
-  let container = getContainerForRawNode(nodeOrSelector, inspector);
-  ok(container, "The container for '" + nodeOrSelector + "' was found");
+  let container = yield getContainerForSelector(selector, inspector);
+  ok(container, "The container for '" + selector + "' was found");
 
   info("Listening for the markupmutation event");
   let nodeMutated = inspector.once("markupmutation");
   setEditableFieldValue(container.editor.newAttr, text, inspector);
-  return nodeMutated;
-}
+  yield nodeMutated;
+});
 
 /**
  * Checks that a node has the given attributes
  *
- * @param {DOMNode|String} nodeOrSelector The node or node selector to check.
+ * @param {String} selector The node or node selector to check.
  * @param {Object} attrs An object containing the attributes to check.
  *        e.g. {id: "id1", class: "someclass"}
  *
  * Note that node.getAttribute() returns attribute values provided by the HTML
  * parser. The parser only provides unescaped entities so &amp; will return &.
  */
-function assertAttributes(nodeOrSelector, attrs) {
-  let node = getNode(nodeOrSelector);
+function assertAttributes(selector, attrs) {
+  let node = getNode(selector);
 
   is(node.attributes.length, Object.keys(attrs).length,
     "Node has the correct number of attributes.");
@@ -381,3 +398,166 @@ function wait(ms) {
   content.setTimeout(def.resolve, ms);
   return def.promise;
 }
+
+/**
+ * Wait for eventName on target.
+ * @param {Object} target An observable object that either supports on/off or
+ * addEventListener/removeEventListener
+ * @param {String} eventName
+ * @param {Boolean} useCapture Optional, for addEventListener/removeEventListener
+ * @return A promise that resolves when the event has been handled
+ */
+function once(target, eventName, useCapture=false) {
+  info("Waiting for event: '" + eventName + "' on " + target + ".");
+
+  let deferred = promise.defer();
+
+  for (let [add, remove] of [
+    ["addEventListener", "removeEventListener"],
+    ["addListener", "removeListener"],
+    ["on", "off"]
+  ]) {
+    if ((add in target) && (remove in target)) {
+      target[add](eventName, function onEvent(...aArgs) {
+        info("Got event: '" + eventName + "' on " + target + ".");
+        target[remove](eventName, onEvent, useCapture);
+        deferred.resolve.apply(deferred, aArgs);
+      }, useCapture);
+      break;
+    }
+  }
+
+  return deferred.promise;
+}
+
+/**
+ * Check to see if the inspector menu items for editing are disabled.
+ * Things like Edit As HTML, Delete Node, etc.
+ * @param {NodeFront} nodeFront
+ * @param {InspectorPanel} inspector
+ * @param {Boolean} assert Should this function run assertions inline.
+ * @return A promise that resolves with a boolean indicating whether
+ *         the menu items are disabled once the menu has been checked.
+ */
+let isEditingMenuDisabled = Task.async(function*(nodeFront, inspector, assert=true) {
+  let deleteMenuItem = inspector.panelDoc.getElementById("node-menu-delete");
+  let editHTMLMenuItem = inspector.panelDoc.getElementById("node-menu-edithtml");
+  let pasteHTMLMenuItem = inspector.panelDoc.getElementById("node-menu-pasteouterhtml");
+
+  // To ensure clipboard contains something to paste.
+  clipboard.set("<p>test</p>", "html");
+
+  let menu = inspector.nodemenu;
+  yield selectNode(nodeFront, inspector);
+  yield reopenMenu(menu);
+
+  let isDeleteMenuDisabled = deleteMenuItem.hasAttribute("disabled");
+  let isEditHTMLMenuDisabled = editHTMLMenuItem.hasAttribute("disabled");
+  let isPasteHTMLMenuDisabled = pasteHTMLMenuItem.hasAttribute("disabled");
+
+  if (assert) {
+    ok(isDeleteMenuDisabled, "Delete menu item is disabled");
+    ok(isEditHTMLMenuDisabled, "Edit HTML menu item is disabled");
+    ok(isPasteHTMLMenuDisabled, "Paste HTML menu item is disabled");
+  }
+
+  return isDeleteMenuDisabled && isEditHTMLMenuDisabled && isPasteHTMLMenuDisabled;
+});
+
+/**
+ * Check to see if the inspector menu items for editing are enabled.
+ * Things like Edit As HTML, Delete Node, etc.
+ * @param {NodeFront} nodeFront
+ * @param {InspectorPanel} inspector
+ * @param {Boolean} assert Should this function run assertions inline.
+ * @return A promise that resolves with a boolean indicating whether
+ *         the menu items are enabled once the menu has been checked.
+ */
+let isEditingMenuEnabled = Task.async(function*(nodeFront, inspector, assert=true) {
+  let deleteMenuItem = inspector.panelDoc.getElementById("node-menu-delete");
+  let editHTMLMenuItem = inspector.panelDoc.getElementById("node-menu-edithtml");
+  let pasteHTMLMenuItem = inspector.panelDoc.getElementById("node-menu-pasteouterhtml");
+
+  // To ensure clipboard contains something to paste.
+  clipboard.set("<p>test</p>", "html");
+
+  let menu = inspector.nodemenu;
+  yield selectNode(nodeFront, inspector);
+  yield reopenMenu(menu);
+
+  let isDeleteMenuDisabled = deleteMenuItem.hasAttribute("disabled");
+  let isEditHTMLMenuDisabled = editHTMLMenuItem.hasAttribute("disabled");
+  let isPasteHTMLMenuDisabled = pasteHTMLMenuItem.hasAttribute("disabled");
+
+  if (assert) {
+    ok(!isDeleteMenuDisabled, "Delete menu item is enabled");
+    ok(!isEditHTMLMenuDisabled, "Edit HTML menu item is enabled");
+    ok(!isPasteHTMLMenuDisabled, "Paste HTML menu item is enabled");
+  }
+
+  return !isDeleteMenuDisabled && !isEditHTMLMenuDisabled && !isPasteHTMLMenuDisabled;
+});
+
+/**
+ * Open a menu (closing it first if necessary).
+ * @param {DOMNode} menu A menu that implements hidePopup/openPopup
+ * @return a promise that resolves once the menu is opened.
+ */
+let reopenMenu = Task.async(function*(menu) {
+  // First close it is if it is already opened.
+  if (menu.state == "closing" || menu.state == "open") {
+    let popuphidden = once(menu, "popuphidden", true);
+    menu.hidePopup();
+    yield popuphidden;
+  }
+
+  // Then open it and return once
+  let popupshown = once(menu, "popupshown", true);
+  menu.openPopup();
+  yield popupshown;
+});
+
+/**
+ * Wait for all current promises to be resolved. See this as executeSoon that
+ * can be used with yield.
+ */
+function promiseNextTick() {
+  let deferred = promise.defer();
+  executeSoon(deferred.resolve);
+  return deferred.promise;
+}
+
+/**
+ * Collapses the current text selection in an input field and tabs to the next
+ * field.
+ */
+function collapseSelectionAndTab(inspector) {
+  EventUtils.sendKey("tab", inspector.panelWin); // collapse selection and move caret to end
+  EventUtils.sendKey("tab", inspector.panelWin); // next element
+}
+
+/**
+ * Collapses the current text selection in an input field and tabs to the
+ * previous field.
+ */
+function collapseSelectionAndShiftTab(inspector) {
+  EventUtils.synthesizeKey("VK_TAB", { shiftKey: true },
+    inspector.panelWin); // collapse selection and move caret to end
+  EventUtils.synthesizeKey("VK_TAB", { shiftKey: true },
+    inspector.panelWin); // previous element
+}
+
+/**
+ * Check that the current focused element is an attribute element in the markup
+ * view.
+ * @param {String} attrName The attribute name expected to be found
+ * @param {Boolean} editMode Whether or not the attribute should be in edit mode
+ */
+function checkFocusedAttribute(attrName, editMode) {
+  let focusedAttr = Services.focus.focusedElement;
+  is(focusedAttr ? focusedAttr.parentNode.dataset.attr : undefined,
+    attrName, attrName + " attribute editor is currently focused.");
+  is(focusedAttr ? focusedAttr.tagName : undefined,
+    editMode ? "input": "span",
+    editMode ? attrName + " is in edit mode" : attrName + " is not in edit mode");
+}
