@@ -19,17 +19,16 @@ const TEST_IMG_BASE64 =
   "A16/JvfiigMSYyzqJXlw/XKUyOORMUaBor6YavgdjKa8xGOnidadmwtwsnMu18q83/kHSou+bFND" +
   "Dr4AAAAASUVORK5CYII=";
 
-let testDriver;
+let testDriver, hud;
 
 function test() {
-  addTab(TEST_URI);
-  browser.addEventListener("load", function onLoad() {
-    browser.removeEventListener("load", onLoad, true);
-    openConsole(null, testNetworkPanel);
-  }, true);
+  loadTab(TEST_URI).then(() => {
+    openConsole().then(testNetworkPanel);
+  });
 }
 
-function testNetworkPanel() {
+function testNetworkPanel(aHud) {
+  hud = aHud;
   testDriver = testGen();
   testDriver.next();
 }
@@ -71,7 +70,6 @@ function checkNodeKeyValue(aPanel, aId, aKey, aValue) {
 }
 
 function testGen() {
-  let hud = HUDService.getHudByWindow(content);
   let filterBox = hud.ui.filterBox;
 
   let httpActivity = {
@@ -107,7 +105,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   info("test 1");
 
@@ -227,7 +225,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: true,
@@ -266,7 +264,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: true,
@@ -301,7 +299,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: true,
@@ -331,7 +329,7 @@ function testGen() {
     networkPanel.panel.hidePopup();
     testDriver.next();
   }, false);
-  yield;
+  yield undefined;
 
   // Check cached image request.
   info("test 8: cached image request");
@@ -347,7 +345,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: true,
@@ -360,7 +358,7 @@ function testGen() {
     responseImageCached: true
   });
 
-  let imgNode = networkPanel.document.getElementById("responseImageCachedNode");
+  imgNode = networkPanel.document.getElementById("responseImageCachedNode");
   is(imgNode.getAttribute("src"), "data:image/png;base64," + TEST_IMG_BASE64,
      "Displayed image is correct");
 
@@ -382,7 +380,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: false,
@@ -411,7 +409,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: false,
@@ -443,7 +441,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: false,
@@ -476,7 +474,7 @@ function testGen() {
     });
   };
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: false,
@@ -512,7 +510,7 @@ function testGen() {
     testDriver.next();
   }
 
-  yield;
+  yield undefined;
 
   checkIsVisible(networkPanel, {
     requestBody: false,
@@ -536,8 +534,8 @@ function testGen() {
   networkPanel.panel.hidePopup(); */
 
   // All done!
-  testDriver = null;
+  testDriver = hud = null;
   executeSoon(finishTest);
 
-  yield;
+  yield undefined;
 }
