@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* vim: set ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
@@ -16,6 +16,6 @@ function test() {
     [" example/a ", {ctrlKey: true}, "http://www.example.com/a"]
   ];
   for (let [input, modifiers, result] of testcases) {
-    is(BrowserUI._canonizeURL(input, modifiers), result, input + " -> " + result);
+    is(BrowserUI._edit._canonizeURL(input, modifiers), result, input + " -> " + result);
   }
 }
