@@ -1,4 +1,4 @@
-// -*- Mode: js2; tab-width: 2; indent-tabs-mode: nil; js2-basic-offset: 2; js2-skip-preprocessor-directives: t; -*-
+// -*- indent-tabs-mode: nil; js-indent-level: 2 -*-
 /* vim: set ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
@@ -31,8 +31,6 @@ function checkContextMenuPositionRange(aElement, aMinLeft, aMaxLeft, aMinTop, aM
 gTests.push({
   desc: "text context menu",
   run: function test() {
-    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
-
     info(chromeRoot + "browser_context_menu_tests_02.html");
     yield addTab(chromeRoot + "browser_context_menu_tests_02.html");
 
@@ -54,11 +52,11 @@ gTests.push({
 
     // invoke selection context menu
     let promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, span, 85, 10);
+    sendContextMenuClickToElement(win, span);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     // selected text context:
     checkContextUIMenuItemVisibility(["context-copy",
@@ -87,11 +85,11 @@ gTests.push({
     let link = win.document.getElementById("text2-link");
     win.getSelection().selectAllChildren(link);
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, link, 40, 10);
+    sendContextMenuClickToElement(win, link);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     // selected text context:
     checkContextUIMenuItemVisibility(["context-copy",
@@ -100,20 +98,22 @@ gTests.push({
                                       "context-copy-link"]);
 
     promise = waitForEvent(document, "popuphidden");
-    ContextMenuUI.hide();
-    yield promise;
+    win.scrollBy(0, 1);
+    let hidden = yield promise;
+    ok(hidden && !(hidden instanceof Error), "scrolling hides the context menu");
     win.getSelection().removeAllRanges();
+    win.scrollBy(0, -1);
 
     ////////////////////////////////////////////////////////////
     // Context menu in content on a link
 
     link = win.document.getElementById("text2-link");
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, link, 40, 10);
+    sendContextMenuClickToElement(win, link);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     // selected text context:
     checkContextUIMenuItemVisibility(["context-open-in-new-tab",
@@ -131,11 +131,11 @@ gTests.push({
 
     let input = win.document.getElementById("text3-input");
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextUIMenuItemVisibility(["context-select",
                                       "context-select-all"]);
@@ -155,11 +155,11 @@ gTests.push({
     input.value = "hello, I'm sorry but I must be going.";
     input.setSelectionRange(0, 5);
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input, 20);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextUIMenuItemVisibility(["context-cut",
                                       "context-copy"]);
@@ -187,11 +187,11 @@ gTests.push({
     input = win.document.getElementById("text3-input");
     input.select();
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input, 20);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     // selected text context:
     checkContextUIMenuItemVisibility(["context-cut",
@@ -208,11 +208,11 @@ gTests.push({
     input = win.document.getElementById("text3-input");
     input.select();
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input, 20);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     // selected text context:
     checkContextUIMenuItemVisibility(["context-cut",
@@ -232,11 +232,11 @@ gTests.push({
     input.value = "hello, I'm sorry but I must be going.";
     input.setSelectionRange(0, 5);
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input, 20);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextUIMenuItemVisibility(["context-cut",
                                       "context-copy"]);
@@ -268,11 +268,11 @@ gTests.push({
     input.value = "";
 
     promise = waitForEvent(document, "popupshown");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input, 20);
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     // selected text context:
     checkContextUIMenuItemVisibility(["context-paste"]);
@@ -291,11 +291,11 @@ gTests.push({
     input.value = "";
 
     promise = waitForEvent(Elements.tray, "transitionend");
-    sendContextMenuClickToElement(win, input, 20, 10);
+    sendContextMenuClickToElement(win, input, 20);
     yield promise;
 
     // should *not* be visible
-    ok(!ContextMenuUI._menuPopup._visible, "is visible");
+    ok(!ContextMenuUI._menuPopup.visible, "is visible");
 
     // the test above will invoke the app bar
     yield hideContextUI();
@@ -308,8 +308,6 @@ gTests.push({
 gTests.push({
   desc: "checks for context menu positioning when browser shifts",
   run: function test() {
-    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
-
     info(chromeRoot + "browser_context_menu_tests_02.html");
     yield addTab(chromeRoot + "browser_context_menu_tests_02.html");
 
@@ -332,17 +330,17 @@ gTests.push({
 
     // invoke selection context menu
     let promise = waitForEvent(document, "popupshown");
-    sendContextMenuClick(225, 310);
+    sendContextMenuClickToElement(browserwin, span);
     yield promise;
 
     // should be visible and at a specific position
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     let notificationBox = Browser.getNotificationBox();
     let notification = notificationBox.getNotificationWithValue("popup-blocked");
     let notificationHeight = notification.boxObject.height;
 
-    checkContextMenuPositionRange(ContextMenuUI._panel, 65, 80, notificationHeight +  155, notificationHeight + 180);
+    checkContextMenuPositionRange(ContextMenuUI._panel, 0, 15, 220, 230);
 
     promise = waitForEvent(document, "popuphidden");
     ContextMenuUI.hide();
@@ -352,6 +350,9 @@ gTests.push({
   }
 });
 
+/*
+XXX code used to diagnose bug 880739
+
 var observeLogger = {
   observe: function (aSubject, aTopic, aData) {
     info("observeLogger: " + aTopic);
@@ -383,15 +384,18 @@ var observeLogger = {
     Services.obs.removeObserver(observeLogger, "dl-cancel");
   }
 }
+*/
 
 // Image context menu tests
 gTests.push({
   desc: "image context menu",
   setUp: function() {
-    observeLogger.init();
+    // XXX code used to diagnose bug 880739
+    //observeLogger.init();
   },
   tearDown: function() {
-    observeLogger.shutdown();
+    // XXX code used to diagnose bug 880739
+    //observeLogger.shutdown();
   },
   run: function test() {
     info(chromeRoot + "browser_context_menu_tests_01.html");
@@ -409,6 +413,8 @@ gTests.push({
 
     ////////////////////////////////////////////////////////////
     // Context menu options
+    /*
+    XXX disabled temporarily due to bug 880739
 
     // image01 - 1x1x100x100
     let promise = waitForEvent(document, "popupshown");
@@ -417,7 +423,7 @@ gTests.push({
 
     purgeEventQueue();
 
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextUIMenuItemVisibility(["context-save-image-lib",
                                       "context-copy-image",
@@ -456,19 +462,19 @@ gTests.push({
     purgeEventQueue();
 
     ok(saveLocationPath.exists(), "image saved");
-
+    */
     ////////////////////////////////////////////////////////////
     // Copy image
 
     let promise = waitForEvent(document, "popupshown");
     sendContextMenuClickToWindow(win, 20, 20);
     yield promise;
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
-    menuItem = document.getElementById("context-copy-image");
+    let menuItem = document.getElementById("context-copy-image");
     ok(menuItem, "menu item exists");
     ok(!menuItem.hidden, "menu item visible");
-    popupPromise = waitForEvent(document, "popuphidden");
+    let popupPromise = waitForEvent(document, "popuphidden");
     EventUtils.synthesizeMouse(menuItem, 10, 10, {}, win);
     yield popupPromise;
 
@@ -484,7 +490,7 @@ gTests.push({
     promise = waitForEvent(document, "popupshown");
     sendContextMenuClickToWindow(win, 30, 30);
     yield promise;
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     menuItem = document.getElementById("context-copy-image-loc");
     ok(menuItem, "menu item exists");
@@ -517,7 +523,7 @@ gTests.push({
     promise = waitForEvent(document, "popupshown");
     sendContextMenuClickToWindow(win, 40, 40);
     yield promise;
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     menuItem = document.getElementById("context-open-image-tab");
     ok(menuItem, "menu item exists");
@@ -562,7 +568,7 @@ gTests.push({
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextMenuPositionRange(ContextMenuUI._panel, 265, 280, 175, 190);
 
@@ -577,7 +583,7 @@ gTests.push({
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextMenuPositionRange(ContextMenuUI._panel, 265, 280, 95, 110);
 
@@ -592,7 +598,7 @@ gTests.push({
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextMenuPositionRange(ContextMenuUI._panel, 295, 310, 540, 555);
 
@@ -607,7 +613,7 @@ gTests.push({
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextMenuPositionRange(ContextMenuUI._panel, 295, 310, 340, 355);
 
@@ -622,16 +628,332 @@ gTests.push({
     yield promise;
 
     // should be visible
-    ok(ContextMenuUI._menuPopup._visible, "is visible");
+    ok(ContextMenuUI._menuPopup.visible, "is visible");
 
     checkContextMenuPositionRange(ContextMenuUI._panel, 265, 280, 110, 125);
 
     promise = waitForEvent(document, "popuphidden");
     ContextMenuUI.hide();
     yield promise;
+
+    Browser.closeTab(Browser.selectedTab, { forceClose: true });
+  }
+});
+
+function reopenSetUp() {
+  info(chromeRoot + "browser_context_menu_tests_04.html");
+  yield addTab(chromeRoot + "browser_context_menu_tests_04.html");
+
+  // Sometimes the context UI won't actually show up.
+  // Since we're just normalizing, we don't want waitForCondition
+  // to cause an orange, so we're putting a try/catch here.
+  try {
+    yield waitForCondition(() => ContextUI.isVisible);
+    ContextUI.dismiss();
+  } catch(e) {}
+}
+
+function reopenTearDown() {
+  let promise = waitForEvent(document, "popuphidden")
+  ContextMenuUI.hide();
+  yield promise;
+  ok(!ContextMenuUI._menuPopup.visible, "popup is actually hidden");
+
+  Browser.closeTab(Browser.selectedTab, { forceClose: true });
+}
+
+function getReopenTest(aElementInputFn, aWindowInputFn) {
+  return function () {
+    let win = Browser.selectedTab.browser.contentWindow;
+    let panel = ContextMenuUI._menuPopup._panel;
+
+    let link1 = win.document.getElementById("text1-link");
+    let link2 = win.document.getElementById("text2-link");
+
+    // Show the menu on link 1
+    let showpromise = waitForEvent(panel, "popupshown");
+    aElementInputFn(win, link1);
+
+    ok((yield showpromise), "popupshown event fired");
+    ok(ContextMenuUI._menuPopup.visible, "initial popup is visible");
+
+    // Show the menu on link 2
+    let hidepromise = waitForEvent(panel, "popuphidden");
+    showpromise = waitForEvent(panel, "popupshown");
+    aElementInputFn(win, link2);
+
+    ok((yield hidepromise), "popuphidden event fired");
+    ok((yield showpromise), "popupshown event fired");
+    ok(ContextMenuUI._menuPopup.visible, "popup is still visible");
+
+    // Hide the menu
+    hidepromise = waitForEvent(panel, "popuphidden")
+    aWindowInputFn(win, 10, 10);
+
+    ok((yield hidepromise), "popuphidden event fired");
+    ok(!ContextMenuUI._menuPopup.visible, "popup is no longer visible");
+  }
+}
+
+gTests.push({
+  desc: "bug 856264 - mouse - context menu should reopen on other links",
+  setUp: reopenSetUp,
+  tearDown: reopenTearDown,
+  run: getReopenTest(sendContextMenuMouseClickToElement, sendMouseClick)
+});
+
+gTests.push({
+  desc: "bug 856264 - touch - context menu should reopen on other links",
+  setUp: reopenSetUp,
+  tearDown: reopenTearDown,
+  run: getReopenTest(sendContextMenuClickToElement, sendTap)
+});
+
+gTests.push({
+  desc: "Bug 947505 - Right-click in a designMode document should display a " +
+        "context menu",
+  run: function test() {
+    info(chromeRoot + "browser_context_menu_tests_02.html");
+    yield addTab(chromeRoot + "browser_context_menu_tests_02.html");
+
+    purgeEventQueue();
+    emptyClipboard();
+    ContextUI.dismiss();
+
+    yield waitForCondition(() => !ContextUI.navbarVisible);
+
+    let tabWindow = Browser.selectedTab.browser.contentWindow;
+    let testSpan = tabWindow.document.getElementById("text1");
+
+    // Case #1: Document isn't in design mode and nothing is selected.
+    tabWindow.document.designMode = "off";
+
+    // Simulate right mouse click to reproduce the same step as noted in the
+    // appropriate bug. It's valid for non-touch case only.
+    synthesizeNativeMouseRDown(Browser.selectedTab.browser, 10, 10);
+    synthesizeNativeMouseRUp(Browser.selectedTab.browser, 10, 10);
+
+    yield waitForCondition(() => ContextUI.navbarVisible);
+
+    ok(ContextUI.navbarVisible, "Navbar is visible on context menu action.");
+    ok(ContextUI.tabbarVisible, "Tabbar is visible on context menu action.");
+
+    ContextUI.dismiss();
+    yield waitForCondition(() => !ContextUI.navbarVisible);
+
+    // Case #2: Document isn't in design mode and text is selected.
+    tabWindow.getSelection().selectAllChildren(testSpan);
+
+    let promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToSelection(tabWindow);
+    yield promise;
+
+    checkContextUIMenuItemVisibility(["context-copy", "context-search"]);
+
+    promise = waitForEvent(document, "popuphidden");
+    ContextMenuUI.hide();
+    yield promise;
+
+    // Case #3: Document is in design mode and nothing is selected.
+    tabWindow.document.designMode = "on";
+    tabWindow.getSelection().removeAllRanges();
+
+    promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToElement(tabWindow, testSpan);
+    yield promise;
+
+    checkContextUIMenuItemVisibility(["context-select-all", "context-select"]);
+
+    promise = waitForEvent(document, "popuphidden");
+    ContextMenuUI.hide();
+    yield promise;
+
+    // Case #4: Document is in design mode and text is selected.
+    tabWindow.getSelection().selectAllChildren(testSpan);
+
+    promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToSelection(tabWindow);
+    yield promise;
+
+    checkContextUIMenuItemVisibility(["context-cut", "context-copy",
+                                      "context-select-all", "context-select",
+                                      "context-search"]);
+
+    promise = waitForEvent(document, "popuphidden");
+    ContextMenuUI.hide();
+    yield promise;
+
+    Browser.closeTab(Browser.selectedTab, { forceClose: true });
+  }
+});
+
+gTests.push({
+  desc: "Bug 961702 - 'Copy' context menu action does not copy rich content " +
+        "while document in design mode (or inside container that allows to " +
+        "edit its content)",
+  run: function test() {
+    info(chromeRoot + "browser_context_menu_tests_05.html");
+    yield addTab(chromeRoot + "browser_context_menu_tests_05.html");
+
+    purgeEventQueue();
+    emptyClipboard();
+    ContextUI.dismiss();
+
+    yield waitForCondition(() => !ContextUI.navbarVisible);
+
+    let tabWindow = Browser.selectedTab.browser.contentWindow;
+    let testDiv = tabWindow.document.getElementById("div1");
+
+    // Case #1: Document is in design mode.
+    tabWindow.document.designMode = "on";
+
+    let promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToElement(tabWindow, testDiv);
+    yield promise;
+
+    let selectAllMenuItem = document.getElementById("context-select-all");
+    promise = waitForEvent(document, "popuphidden");
+    sendNativeTap(selectAllMenuItem);
+    yield promise;
+
+    promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToSelection(tabWindow);
+    yield promise;
+
+    let copyMenuItem = document.getElementById("context-copy");
+    promise = waitForEvent(document, "popuphidden");
+    sendNativeTap(copyMenuItem);
+    yield promise;
+
+    // The wait is needed to give time to populate the clipboard.
+    let clipboardContent = "";
+    let contentToCopy = tabWindow.document.body.innerHTML;
+    yield waitForCondition(function () {
+      clipboardContent = SpecialPowers.getClipboardData("text/html");
+      return clipboardContent == contentToCopy;
+    });
+    ok(clipboardContent == contentToCopy, "Rich content copied.");
+
+    // Case #2: Container with editable content.
+    emptyClipboard();
+    tabWindow.document.designMode = "off";
+    tabWindow.getSelection().removeAllRanges();
+
+    promise = waitForEvent(tabWindow.document.body, "focus");
+    sendNativeTap(testDiv);
+    yield promise;
+
+    promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToElement(tabWindow, testDiv);
+    yield promise;
+
+    selectAllMenuItem = document.getElementById("context-select-all");
+    promise = waitForEvent(document, "popuphidden");
+    sendNativeTap(selectAllMenuItem);
+    yield promise;
+
+    promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToSelection(tabWindow);
+    yield promise;
+
+    copyMenuItem = document.getElementById("context-copy");
+    promise = waitForEvent(document, "popuphidden");
+    sendNativeTap(copyMenuItem);
+    yield promise;
+
+     // The wait is needed to give time to populate the clipboard.
+    clipboardContent = "";
+    contentToCopy = testDiv.innerHTML;
+    yield waitForCondition(function () {
+      clipboardContent = SpecialPowers.getClipboardData("text/html");
+      return clipboardContent == contentToCopy;
+    });
+    ok(clipboardContent == contentToCopy, "Rich content copied.");
+
+    Browser.closeTab(Browser.selectedTab, { forceClose: true });
+  }
+});
+
+gTests.push({
+  desc: "Bug 963067 - 'Cut' in the cut, copy, paste menu is always active " +
+        "after a browser launch.",
+  run: function test() {
+    info(chromeRoot + "browser_context_menu_tests_02.html");
+    yield addTab(chromeRoot + "browser_context_menu_tests_02.html");
+
+    purgeEventQueue();
+    emptyClipboard();
+
+    ContextUI.dismiss();
+    yield waitForCondition(() => !ContextUI.navbarVisible);
+
+    let tabWindow = Browser.selectedTab.browser.contentWindow;
+    let input = tabWindow.document.getElementById("text3-input");
+    let cutMenuItem = document.getElementById("context-cut");
+
+    input.select();
+
+    // Emulate RichListBox's behavior and set first item selected by default.
+    cutMenuItem.selected = true;
+
+    let promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToElement(tabWindow, input);
+    yield promise;
+
+    ok(!cutMenuItem.hidden && !cutMenuItem.selected,
+       "Cut menu item is visible and not selected.");
+
+    promise = waitForEvent(document, "popuphidden");
+    ContextMenuUI.hide();
+    yield promise;
+
+    Browser.closeTab(Browser.selectedTab, { forceClose: true });
+  }
+});
+
+gTests.push({
+  desc: "Bug 867499 - Selecting 'copy' from context menu for selected text " +
+        "dismisses selection.",
+  run: function test() {
+    info(chromeRoot + "browser_context_menu_tests_02.html");
+    yield addTab(chromeRoot + "browser_context_menu_tests_02.html");
+
+    emptyClipboard();
+    ContextUI.dismiss();
+
+    yield waitForCondition(() => !ContextUI.navbarVisible);
+
+    let tabWindow = Browser.selectedTab.browser.contentWindow;
+    let testSpan = tabWindow.document.getElementById("text1");
+
+    let promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToElement(tabWindow, testSpan, 5, 5);
+    yield promise;
+
+    yield waitForCondition(()=>SelectionHelperUI.isSelectionUIVisible);
+
+    promise = waitForEvent(document, "popupshown");
+    sendContextMenuClickToSelection(tabWindow);
+    yield promise;
+
+    let copyMenuItem = document.getElementById("context-copy");
+
+    ok(!copyMenuItem.hidden, "Copy menu item should be visible.");
+
+    promise = waitForEvent(document, "popuphidden");
+    sendNativeTap(copyMenuItem, 5, 5);
+    yield promise;
+    yield waitForCondition(() =>
+        !!SpecialPowers.getClipboardData("text/unicode"));
+
+    ok(SelectionHelperUI.isSelectionUIVisible,
+        "Selection monocles should stay active after copy action.");
+
+    Browser.closeTab(Browser.selectedTab, { forceClose: true });
   }
 });
 
 function test() {
+  setDevPixelEqualToPx();
   runTests();
 }
