@@ -6,7 +6,9 @@
 
 #include "imgIContainer.h"
 #include "ClippedImage.h"
+#include "DynamicImage.h"
 #include "FrozenImage.h"
+#include "OrientedImage.h"
 #include "Image.h"
 
 #include "ImageOps.h"
@@ -44,5 +46,27 @@ ImageOps::Clip(imgIContainer* aImage, nsIntRect aClip)
   return clippedImage.forget();
 }
 
+/* static */ already_AddRefed<Image>
+ImageOps::Orient(Image* aImage, Orientation aOrientation)
+{
+  nsRefPtr<Image> orientedImage = new OrientedImage(aImage, aOrientation);
+  return orientedImage.forget();
+}
+
+/* static */ already_AddRefed<imgIContainer>
+ImageOps::Orient(imgIContainer* aImage, Orientation aOrientation)
+{
+  nsCOMPtr<imgIContainer> orientedImage =
+    new OrientedImage(static_cast<Image*>(aImage), aOrientation);
+  return orientedImage.forget();
+}
+
+/* static */ already_AddRefed<imgIContainer>
+ImageOps::CreateFromDrawable(gfxDrawable* aDrawable)
+{
+  nsCOMPtr<imgIContainer> drawableImage = new DynamicImage(aDrawable);
+  return drawableImage.forget();
+}
+
 } // namespace image
 } // namespace mozilla
