@@ -6,8 +6,9 @@
 #ifndef ctypes_Library_h
 #define ctypes_Library_h
 
-#include "jsapi.h"
+#include "js/TypeDecls.h"
 
+struct JSCTypesCallbacks;
 struct PRLibrary;
 
 namespace js {
@@ -20,14 +21,14 @@ enum LibrarySlot {
 
 namespace Library
 {
-  JSBool Name(JSContext* cx, unsigned argc, jsval *vp);
+  bool Name(JSContext* cx, unsigned argc, JS::Value* vp);
 
-  JSObject* Create(JSContext* cx, jsval path, JSCTypesCallbacks* callbacks);
+  JSObject* Create(JSContext* cx, JS::Value path, const JSCTypesCallbacks* callbacks);
 
   bool IsLibrary(JSObject* obj);
   PRLibrary* GetLibrary(JSObject* obj);
 
-  JSBool Open(JSContext* cx, unsigned argc, jsval* vp);
+  bool Open(JSContext* cx, unsigned argc, JS::Value* vp);
 }
 
 }
