@@ -1,7 +1,12 @@
 // obj.getOwnPropertyNames() works when obj's referent is itself a cross-compartment wrapper.
 
-var g = newGlobal('new-compartment');
+var g = newGlobal();
 var dbg = Debugger();
 var gobj = dbg.addDebuggee(g);
 g.p = {xyzzy: 8};  // makes a cross-compartment wrapper
-assertEq(gobj.getOwnPropertyDescriptor("p").value.getOwnPropertyDescriptor("xyzzy").value, 8);
+g.p[Symbol.for("plugh")] = 9;
+var wp = gobj.getOwnPropertyDescriptor("p").value;
+var names = wp.getOwnPropertyNames();
+assertEq(names.length, 1);
+assertEq(names[0], "xyzzy");
+
