@@ -12,28 +12,28 @@ BEGIN_TEST(testIntString_bug515273)
     JS::RootedValue v(cx);
 
     EVAL("'1';", &v);
-    JSString* str = JSVAL_TO_STRING(v);
+    JSString* str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "1"));
 
     EVAL("'42';", &v);
-    str = JSVAL_TO_STRING(v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "42"));
 
     EVAL("'111';", &v);
-    str = JSVAL_TO_STRING(v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "111"));
 
     /* Test other types of static strings. */
     EVAL("'a';", &v);
-    str = JSVAL_TO_STRING(v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "a"));
 
     EVAL("'bc';", &v);
-    str = JSVAL_TO_STRING(v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "bc"));
 
