@@ -1,11 +1,11 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 
 /**
-   File Name:          7.4.3-8-n.js
+   File Name:          7.4.3-2-n.js
    ECMA Section:       7.4.3
 
    Description:
@@ -22,16 +22,16 @@
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
-var SECTION = "7.4.3-9-n";
+var SECTION = "7.4.3-2-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Future Reserved Words";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var switch = true";
+DESCRIPTION = "var debugger = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var switch = true",     "error",    eval("var switch = true") );
+new TestCase( SECTION,  "var debugger = true",     "error",    eval("var debugger = true") );
 
 test();
