@@ -1,5 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- *
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*-
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -44,6 +43,17 @@ var DEBUG = false;
 var DESCRIPTION;
 var EXPECTED;
 
+/*
+ * Signals to results.py that the current test case should be considered to
+ * have passed if it doesn't throw an exception.
+ *
+ * When the test suite is run in the browser, this function gets overridden by
+ * the same-named function in browser.js.
+ */
+function testPassesUnlessItThrows() {
+  print(PASSED);
+}
+
 /*
  * wrapper for test case constructor that doesn't require the SECTION
  * argument.
@@ -132,7 +142,7 @@ function reportFailure (msg)
   var l;
   var funcName = currentFunc();
   var prefix = (funcName) ? "[reported from " + funcName + "] ": "";
-   
+
   for (var i=0; i<lines.length; i++)
     print (FAILED + prefix + lines[i]);
 }
@@ -165,14 +175,7 @@ function printBugNumber (num)
 
 function toPrinted(value)
 {
-  if (typeof value == "xml") 
-  {
-    value = value.toXMLString();
-  } 
-  else 
-  {
-    value = String(value);
-  }
+  value = String(value);
   value = value.replace(/\\n/g, 'NL')
                .replace(/\n/g, 'NL')
                .replace(/\\r/g, 'CR')
@@ -383,7 +386,7 @@ function enterFunc (funcName)
 function exitFunc (funcName)
 {
   var lastFunc = callStack.pop();
-   
+
   if (funcName)
   {
     if (!funcName.match(/\(\)$/))
@@ -493,7 +496,7 @@ function BigO(data)
     {
       var Ydiff = Y[i] - this.Yavg;
       var Xdiff = X[i] - this.Xavg;
-       
+
       SUM_Ydiff2 += Ydiff * Ydiff;
       SUM_Xdiff2 += Xdiff * Xdiff;
       SUM_XdiffYdiff += Xdiff * Ydiff;
@@ -529,7 +532,7 @@ function BigO(data)
     {
       deriv.X[i] = (X[i] + X[i+1])/2;
       deriv.Y[i] = (Y[i+1] - Y[i])/(X[i+1] - X[i]);
-    } 
+    }
     return deriv;
   }
 
@@ -605,7 +608,7 @@ function optionsInit() {
 }
 
 function optionsClear() {
-       
+
   // turn off current settings
   // except jit.
   var optionNames = options().split(',');
@@ -614,7 +617,8 @@ function optionsClear() {
     var optionName = optionNames[i];
     if (optionName &&
         optionName != "methodjit" &&
-        optionName != "methodjit_always")
+        optionName != "methodjit_always" &&
+        optionName != "ion")
     {
       options(optionName);
     }
@@ -679,7 +683,6 @@ if (typeof options == 'function')
 {
   optionsInit();
   optionsClear();
-  options("allow_xml");
 }
 
 function getTestCaseResult(expected, actual)
@@ -832,6 +835,45 @@ function jit(on)
 {
 }
 
+function assertEqArray(a1, a2) {
+  assertEq(a1.length, a2.length);
+  for (var i = 0; i < a1.length; i++) {
+    try {
+      assertEq(a1[i], a2[i]);
+    } catch (e) {
+      throw new Error("At index " + i + ": " + e);
+    }
+  }
+}
+
+function assertThrows(f) {
+    var ok = false;
+    try {
+        f();
+    } catch (exc) {
+        ok = true;
+    }
+    if (!ok)
+        throw new Error("Assertion failed: " + f + " did not throw as expected");
+}
+
+
+function assertThrowsInstanceOf(f, ctor, msg) {
+  var fullmsg;
+  try {
+    f();
+  } catch (exc) {
+    if (exc instanceof ctor)
+      return;
+    fullmsg = "Assertion failed: expected exception " + ctor.name + ", got " + exc;
+  }
+  if (fullmsg === undefined)
+    fullmsg = "Assertion failed: expected exception " + ctor.name + ", no exception thrown";
+  if (msg !== undefined)
+    fullmsg += " - " + msg;
+  throw new Error(fullmsg);
+};
+
 /*
  * Some tests need to know if we are in Rhino as opposed to SpiderMonkey
  */
@@ -851,5 +893,3 @@ function OptLevel( i ) {
   cx.setOptimizationLevel(i);
 }
 /* end of Rhino functions */
-
-
