@@ -4,17 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.6/7.6.1/7.6.1.2/7.6.1-18-s.js
- * @description 7.6 - SyntaxError expected: reserved words used as Identifier Names in UTF8: l\u0065t (let)
- * @onlyStrict 
+ * @path ch07/7.6/7.6.1/7.6.1.2/7.6.1.2-8-s.js
+ * @description Strict Mode - SyntaxError is thrown when FutureReservedWord 'protected' occurs in strict mode code
+ * @onlyStrict
  */
 
 
-function testcase() {        
+
+
+function testcase() {
         "use strict";
 
         try {
-            eval("var l\u0065t = 123;");
+            eval("var protected = 1;");
             return false;
         } catch (e) {
             return e instanceof SyntaxError;
