@@ -4,7 +4,7 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.8/7.8.4/7.8.4-4-s.js
+ * @path ch07/7.8/7.8.4/7.8.4-31-s.js
  * @description An OctalEscapeSequence is not allowed in a String under Strict Mode
  * @onlyStrict
  */
@@ -14,7 +14,7 @@ function testcase()
 {
   try 
   {
-    eval('"use strict"; var x = "z\\7";');
+    eval('"use strict"; var x = "\\" + "1";');
     return false;
   }
   catch (e) {
