@@ -4,13 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-17-s.js
- * @description Strict Mode - checking 'this' (eval used within strict mode)
+ * @path ch10/10.4/10.4.3/10.4.3-1-7-s.js
+ * @description Strict Mode - checking 'this' (FunctionDeclaration defined within strict mode)
  * @onlyStrict
  */
-    
+
+
 function testcase() {
 "use strict";
-return (eval("typeof this") === "undefined") && (eval("this") !== fnGlobalObject());
+function f() {
+    return typeof this;
 }
-runTestCase(testcase);
\ No newline at end of file
+return f() === "undefined";
+}
+runTestCase(testcase);
