@@ -4,12 +4,13 @@
 /// "Use Terms").   Any redistribution of this code must retain the above
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-66gs.js
- * @description Strict - checking 'this' from a global scope (strict function declaration called by Function.prototype.apply())
+ * @path ch10/10.4/10.4.3/10.4.3-1-98gs.js
+ * @description Strict - checking 'this' from a global scope (non-strict function declaration called by strict Function.prototype.bind(someObject)())
  * @onlyStrict
  */
 
-function f() { "use strict"; return this===undefined;};
-if (! f.apply()){
+var o = {};
+function f() { return this===o;};
+if (! ((function () {"use strict"; return f.bind(o)();})())){
     throw "'this' had incorrect value!";
-}
\ No newline at end of file
+}
