@@ -4,11 +4,11 @@
 /// "Use Terms").   Any redistribution of this code must retain the above
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.2/12.2.1/12.2.1-1gs.js
- * @description Strict Mode - SyntaxError is thrown if a VariableDeclaration occurs within strict code and its Identifier is eval
+ * @path ch12/12.2/12.2.1/12.2.1-4gs.js
+ * @description Strict Mode - SyntaxError is thrown if a VariableDeclarationNoIn occurs within strict code and its Identifier is arguments
  * @onlyStrict
  * @negative ^((?!NotEarlyError).)*$
  */
 "use strict";
 throw NotEarlyError;
-for (var eval in arrObj) { }
\ No newline at end of file
+var arguments;
\ No newline at end of file
