@@ -33,18 +33,19 @@ static nsresult GetExtensionFromWindowsMimeDatabase(const nsACString& aMimeType,
 
 nsOSHelperAppService::nsOSHelperAppService() : 
   nsExternalHelperAppService()
-  , mAppAssoc(nsnull)
+  , mAppAssoc(nullptr)
 {
-  CoInitialize(NULL);
-  CoCreateInstance(CLSID_ApplicationAssociationRegistration, NULL, CLSCTX_INPROC,
-                   IID_IApplicationAssociationRegistration, (void**)&mAppAssoc);
+  CoInitialize(nullptr);
+  CoCreateInstance(CLSID_ApplicationAssociationRegistration, nullptr,
+                   CLSCTX_INPROC, IID_IApplicationAssociationRegistration,
+                   (void**)&mAppAssoc);
 }
 
 nsOSHelperAppService::~nsOSHelperAppService()
 {
   if (mAppAssoc)
     mAppAssoc->Release();
-  mAppAssoc = nsnull;
+  mAppAssoc = nullptr;
   CoUninitialize();
 }
 
@@ -97,12 +98,12 @@ static nsresult GetExtensionFrom4xRegistryInfo(const nsACString& aMimeType,
   if (NS_FAILED(rv))
     return NS_OK;
 
-  aFileExtension.Insert(PRUnichar('.'), 0);
+  aFileExtension.Insert(char16_t('.'), 0);
       
   // this may be a comma separated list of extensions...just take the 
   // first one for now...
 
-  PRInt32 pos = aFileExtension.FindChar(PRUnichar(','));
+  int32_t pos = aFileExtension.FindChar(char16_t(','));
   if (pos > 0) {
     // we have a comma separated list of types...
     // truncate everything after the first comma (including the comma)
@@ -120,7 +121,7 @@ nsresult nsOSHelperAppService::OSProtocolHandlerExists(const char * aProtocolSch
   {
     // Vista: use new application association interface
     if (mAppAssoc) {
-      PRUnichar * pResult = nsnull;
+      wchar_t * pResult = nullptr;
       NS_ConvertASCIItoUTF16 scheme(aProtocolScheme);
       // We are responsible for freeing returned strings.
       HRESULT hr = mAppAssoc->QueryCurrentDefault(scheme.get(),
@@ -141,7 +142,8 @@ nsresult nsOSHelperAppService::OSProtocolHandlerExists(const char * aProtocolSch
                                &hKey);
     if (err == ERROR_SUCCESS)
     {
-      err = ::RegQueryValueExW(hKey, L"URL Protocol", NULL, NULL, NULL, NULL);
+      err = ::RegQueryValueExW(hKey, L"URL Protocol",
+                               nullptr, nullptr, nullptr, nullptr);
       *aHandlerExists = (err == ERROR_SUCCESS);
       // close the key
       ::RegCloseKey(hKey);
@@ -162,7 +164,7 @@ NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(const nsACString&
 
   // Vista: use new application association interface
   if (mAppAssoc) {
-    PRUnichar * pResult = nsnull;
+    wchar_t * pResult = nullptr;
     // We are responsible for freeing returned strings.
     HRESULT hr = mAppAssoc->QueryCurrentDefault(buf.get(),
                                                 AT_URLPROTOCOL, AL_EFFECTIVE,
@@ -236,13 +238,13 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromRegistry(const nsAFlatString& file
 
 /// Looks up the type for the extension aExt and compares it to aType
 /* static */ bool
-nsOSHelperAppService::typeFromExtEquals(const PRUnichar* aExt, const char *aType)
+nsOSHelperAppService::typeFromExtEquals(const char16_t* aExt, const char *aType)
 {
   if (!aType)
     return false;
   nsAutoString fileExtToUse;
-  if (aExt[0] != PRUnichar('.'))
-    fileExtToUse = PRUnichar('.');
+  if (aExt[0] != char16_t('.'))
+    fileExtToUse = char16_t('.');
 
   fileExtToUse.Append(aExt);
 
@@ -276,14 +278,14 @@ static void CleanupHandlerPath(nsString& aPath)
   // 3) C:\Windows\some.dll,-foo -bar
   // 4) C:\Windows\some.cpl,-foo -bar
 
-  PRInt32 lastCommaPos = aPath.RFindChar(',');
+  int32_t lastCommaPos = aPath.RFindChar(',');
   if (lastCommaPos != kNotFound)
     aPath.Truncate(lastCommaPos);
 
-  aPath.AppendLiteral(" ");
+  aPath.Append(' ');
 
   // case insensitive
-  PRUint32 index = aPath.Find(".exe ", true);
+  uint32_t index = aPath.Find(".exe ", true);
   if (index == kNotFound)
     index = aPath.Find(".dll ", true);
   if (index == kNotFound)
@@ -308,15 +310,15 @@ static void StripRundll32(nsString& aCommandString)
   NS_NAMED_LITERAL_STRING(rundllSegmentShort, "rundll32 ");
 
   // case insensitive
-  PRInt32 strLen = rundllSegment.Length();
-  PRInt32 index = aCommandString.Find(rundllSegment, true);
+  int32_t strLen = rundllSegment.Length();
+  int32_t index = aCommandString.Find(rundllSegment, true);
   if (index == kNotFound) {
     strLen = rundllSegmentShort.Length();
     index = aCommandString.Find(rundllSegmentShort, true);
   }
 
   if (index != kNotFound) {
-    PRUint32 rundllSegmentLength = index + strLen;
+    uint32_t rundllSegmentLength = index + strLen;
     aCommandString.Cut(0, rundllSegmentLength);
   }
 }
@@ -345,19 +347,19 @@ static void StripRundll32(nsString& aCommandString)
   //    Viewer.dll", var var
 
   // Expand environment variables so we have full path strings.
-  PRUint32 bufLength = ::ExpandEnvironmentStringsW(handlerCommand.get(),
+  uint32_t bufLength = ::ExpandEnvironmentStringsW(handlerCommand.get(),
                                                    L"", 0);
   if (bufLength == 0) // Error
     return false;
 
-  nsAutoArrayPtr<PRUnichar> destination(new PRUnichar[bufLength]);
+  nsAutoArrayPtr<wchar_t> destination(new wchar_t[bufLength]);
   if (!destination)
     return false;
   if (!::ExpandEnvironmentStringsW(handlerCommand.get(), destination,
                                    bufLength))
     return false;
 
-  handlerCommand = destination;
+  handlerCommand = static_cast<const wchar_t*>(destination);
 
   // Remove quotes around paths
   handlerCommand.StripChars("\"");
@@ -388,7 +390,7 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
   // If all else fails, use the file type key name, which will be 
   // something like "pngfile" for .pngs, "WMVFile" for .wmvs, etc. 
   aDefaultDescription = aAppInfo;
-  *aDefaultApplication = nsnull;
+  *aDefaultApplication = nullptr;
 
   if (aAppInfo.IsEmpty())
     return NS_ERROR_FAILURE;
@@ -475,7 +477,7 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
   if (!lf)
     return NS_ERROR_FILE_NOT_FOUND;
 
-  nsILocalFileWin* lfw = nsnull;
+  nsILocalFileWin* lfw = nullptr;
   CallQueryInterface(lf, &lfw);
 
   if (lfw) {
@@ -491,13 +493,13 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
 already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFlatString& aFileExt, const char *aTypeHint)
 {
   if (aFileExt.IsEmpty())
-    return nsnull;
+    return nullptr;
 
   // windows registry assumes your file extension is going to include the '.'.
   // so make sure it's there...
   nsAutoString fileExtToUse;
-  if (aFileExt.First() != PRUnichar('.'))
-    fileExtToUse = PRUnichar('.');
+  if (aFileExt.First() != char16_t('.'))
+    fileExtToUse = char16_t('.');
 
   fileExtToUse.Append(aFileExt);
 
@@ -505,15 +507,15 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
   nsCOMPtr<nsIWindowsRegKey> regKey = 
     do_CreateInstance("@mozilla.org/windows-registry-key;1");
   if (!regKey) 
-    return nsnull; 
+    return nullptr; 
 
   nsresult rv = regKey->Open(nsIWindowsRegKey::ROOT_KEY_CLASSES_ROOT,
                              fileExtToUse,
                              nsIWindowsRegKey::ACCESS_QUERY_VALUE);
   if (NS_FAILED(rv))
-    return nsnull; 
+    return nullptr; 
 
-  nsCAutoString typeToUse;
+  nsAutoCString typeToUse;
   if (aTypeHint && *aTypeHint) {
     typeToUse.Assign(aTypeHint);
   }
@@ -521,17 +523,13 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
     nsAutoString temp;
     if (NS_FAILED(regKey->ReadStringValue(NS_LITERAL_STRING("Content Type"),
                   temp)) || temp.IsEmpty()) {
-      return nsnull; 
+      return nullptr; 
     }
     // Content-Type is always in ASCII
     LossyAppendUTF16toASCII(temp, typeToUse);
   }
 
-  nsMIMEInfoWin* mimeInfo = new nsMIMEInfoWin(typeToUse);
-  if (!mimeInfo)
-    return nsnull; // out of memory
-
-  NS_ADDREF(mimeInfo);
+  nsRefPtr<nsMIMEInfoWin> mimeInfo = new nsMIMEInfoWin(typeToUse);
 
   // don't append the '.'
   mimeInfo->AppendExtension(NS_ConvertUTF16toUTF8(Substring(fileExtToUse, 1)));
@@ -545,7 +543,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
     // Vista: use the new application association COM interfaces
     // for resolving helpers.
     nsString assocType(fileExtToUse);
-    PRUnichar * pResult = nsnull;
+    wchar_t * pResult = nullptr;
     HRESULT hr = mAppAssoc->QueryCurrentDefault(assocType.get(),
                                                 AT_FILEEXTENSION, AL_EFFECTIVE,
                                                 &pResult);
@@ -569,8 +567,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
     found = false;
 
   if (!found) {
-    NS_IF_RELEASE(mimeInfo); // we failed to really find an entry in the registry
-    return nsnull;
+    return nullptr;
   }
 
   // Get other nsIMIMEInfo fields from registry, if possible.
@@ -579,8 +576,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
   
   if (NS_FAILED(GetDefaultAppInfo(appInfo, defaultDescription,
                                   getter_AddRefs(defaultApplication)))) {
-    NS_IF_RELEASE(mimeInfo);
-    return nsnull;
+    return nullptr;
   }
 
   mimeInfo->SetDefaultDescription(defaultDescription);
@@ -589,7 +585,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
   // Grab the general description
   GetMIMEInfoFromRegistry(appInfo, mimeInfo);
 
-  return mimeInfo;
+  return mimeInfo.forget();
 }
 
 already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType, const nsACString& aFileExt, bool *aFound)
@@ -619,10 +615,10 @@ already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsAC
     }
   }
   // If we found an extension for the type, do the lookup
-  nsMIMEInfoWin* mi = nsnull;
+  nsRefPtr<nsMIMEInfoWin> mi;
   if (!fileExtension.IsEmpty())
-    mi = GetByExtension(fileExtension, flatType.get()).get();
-  LOG(("Extension lookup on '%s' found: 0x%p\n", fileExtension.get(), mi));
+    mi = GetByExtension(fileExtension, flatType.get());
+  LOG(("Extension lookup on '%s' found: 0x%p\n", fileExtension.get(), mi.get()));
 
   bool hasDefault = false;
   if (mi) {
@@ -646,21 +642,18 @@ already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsAC
       GetByExtension(NS_ConvertUTF8toUTF16(aFileExt), flatType.get());
     LOG(("Ext. lookup for '%s' found 0x%p\n", flatExt.get(), miByExt.get()));
     if (!miByExt && mi)
-      return mi;
+      return mi.forget();
     if (miByExt && !mi) {
-      miByExt.swap(mi);
-      return mi;
+      return miByExt.forget();
     }
     if (!miByExt && !mi) {
       *aFound = false;
       mi = new nsMIMEInfoWin(flatType);
-      if (mi) {
-        NS_ADDREF(mi);
-        if (!aFileExt.IsEmpty())
-          mi->AppendExtension(aFileExt);
+      if (!aFileExt.IsEmpty()) {
+        mi->AppendExtension(aFileExt);
       }
       
-      return mi;
+      return mi.forget();
     }
 
     // if we get here, mi has no default app. copy from extension lookup.
@@ -670,7 +663,7 @@ already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsAC
 
     mi->SetDefaultDescription(desc);
   }
-  return mi;
+  return mi.forget();
 }
 
 NS_IMETHODIMP
